@@ -10,13 +10,13 @@ from openmdao.core.constants import INT_DTYPE
 from openmdao.core.explicitcomponent import ExplicitComponent
 from openmdao.utils.units import valid_units
 from openmdao.utils import cs_safe
-from openmdao.warnings import issue_warning, DerivativesWarning, warn_deprecation
+from openmdao.utils.om_warnings import issue_warning, DerivativesWarning, warn_deprecation
 
 # regex to check for variable names.
 VAR_RGX = re.compile(r'([.]*[_a-zA-Z]\w*[ ]*\(?)')
 
 # Names of metadata entries allowed for ExecComp variables.
-_allowed_meta = {'value', 'shape', 'units', 'res_units', 'desc',
+_allowed_meta = {'value', 'val', 'shape', 'units', 'res_units', 'desc',
                  'ref', 'ref0', 'res_ref', 'lower', 'upper', 'src_indices',
                  'flat_src_indices', 'tags', 'shape_by_conn', 'copy_shape'}
 
@@ -31,9 +31,9 @@ def check_option(option, value):
     Parameters
     ----------
     option : str
-        The name of the option
+        The name of the option.
     value : any
-        The value of the option
+        The value of the option.
 
     Raises
     ------
@@ -50,7 +50,11 @@ def array_idx_iter(shape):
     Parameters
     ----------
     shape : tuple
-        shape of the array.
+        Shape of the array.
+
+    Yields
+    ------
+    int
     """
     for p in product(*[range(s) for s in shape]):
         yield p
@@ -60,6 +64,21 @@ class ExecComp(ExplicitComponent):
     """
     A component defined by an expression string.
 
+    Parameters
+    ----------
+    exprs : str, tuple of str or list of str
+        An assignment statement or iter of them. These express how the
+        outputs are calculated based on the inputs.  In addition to
+        standard Python operators, a subset of numpy and scipy functions
+        is supported.
+
+    **kwargs : dict of named args
+        Initial values of variables can be set by setting a named
+        arg with the var name.  If the value is a dict it is assumed
+        to contain metadata.  To set the initial value in addition to
+        other metadata, assign the initial value to the 'val' entry
+        of the dict.
+
     Attributes
     ----------
     _kwargs : dict of named args
@@ -68,6 +87,8 @@ class ExecComp(ExplicitComponent):
         List of expressions.
     _codes : list
         List of code objects.
+    _exprs_info : list
+        List of tuples containing output and inputs for each expression.
     _has_diag_partials : bool
         If True, treat all array/array partials as diagonal if both arrays have size > 1.
         All arrays with size > 1 must have the same flattened size or an exception will be raised.
@@ -149,21 +170,6 @@ class ExecComp(ExplicitComponent):
         zeros(N)                   Create an array of zeros
         =========================  ====================================
 
-        Parameters
-        ----------
-        exprs : str, tuple of str or list of str
-            An assignment statement or iter of them. These express how the
-            outputs are calculated based on the inputs.  In addition to
-            standard Python operators, a subset of numpy and scipy functions
-            is supported.
-
-        **kwargs : dict of named args
-            Initial values of variables can be set by setting a named
-            arg with the var name.  If the value is a dict it is assumed
-            to contain metadata.  To set the initial value in addition to
-            other metadata, assign the initial value to the 'value' entry
-            of the dict.
-
         Notes
         -----
         If a variable has an initial value that is anything other than 1.0,
@@ -189,7 +195,7 @@ class ExecComp(ExplicitComponent):
         .. code-block:: python
 
             excomp = ExecComp('y=sum(x)',
-                              x={'value': numpy.ones(10, dtype=float),
+                              x={'val': numpy.ones(10, dtype=float),
                                  'units': 'ft'})
         """
         options = {}
@@ -206,7 +212,8 @@ class ExecComp(ExplicitComponent):
             exprs = [exprs]
 
         self._exprs = exprs[:]
-        self._codes = None
+        self._exprs_info = []
+        self._codes = []
         self._kwargs = kwargs
 
         self._manual_decl_partials = False
@@ -274,10 +281,8 @@ class ExecComp(ExplicitComponent):
         """
         Set up variable name and metadata lists.
         """
-        if not self._exprs:
-            raise RuntimeError("%s: No valid expressions provided to ExecComp(): %s."
-                               % (self.msginfo, self._exprs))
-        self._setup_expressions()
+        if self._exprs:
+            self._setup_expressions()
 
     def _setup_expressions(self):
         """
@@ -294,6 +299,8 @@ class ExecComp(ExplicitComponent):
         shape = self.options['shape']
         shape_by_conn = self.options['shape_by_conn']
 
+        warned = False
+
         if shape is not None and shape_by_conn:
             raise RuntimeError(f"{self.msginfo}: Can't set both shape and shape_by_conn.")
 
@@ -306,7 +313,7 @@ class ExecComp(ExplicitComponent):
         self._requires_fd = {}
 
         # find all of the variables and which ones are outputs
-        for i, (onames, names) in enumerate(exprs_info):
+        for onames, names in exprs_info:
             outs.update(onames)
             allvars.update(names[0])
             if _not_complex_safe.intersection(names[1]):
@@ -339,6 +346,17 @@ class ExecComp(ExplicitComponent):
                                        "recognized for variable '%s': %s" %
                                        (self.msginfo, arg, sorted(diff)))
 
+                if 'val' in val and 'value' in val:
+                    raise RuntimeError(f"{self.msginfo}: 'val' and 'value' at the same time, use "
+                                       "'val'.")
+                elif 'value' in val and not warned:
+                    warn_deprecation(f"{self.msginfo}: 'value' will be deprecated in 4.0. Please "
+                                     "use 'val' in the future.")
+
+                if 'value' in val:
+                    val['val'] = val.pop('value')
+                    warned = True
+
                 kwargs2[arg] = val.copy()
 
                 if units is not None:
@@ -356,22 +374,22 @@ class ExecComp(ExplicitComponent):
                                            "variable '%s', but shape of %s has been "
                                            "specified for the entire component." %
                                            (self.msginfo, val['shape'], arg, shape))
-                    elif 'value' in val and np.atleast_1d(val['value']).shape != shape:
+                    elif 'val' in val and np.atleast_1d(val['val']).shape != shape:
                         raise RuntimeError("%s: value of shape %s has been specified for "
                                            "variable '%s', but shape of %s has been "
                                            "specified for the entire component." %
-                                           (self.msginfo, np.atleast_1d(val['value']).shape,
+                                           (self.msginfo, np.atleast_1d(val['val']).shape,
                                             arg, shape))
                     else:
                         init_vals[arg] = np.ones(shape)
 
-                if 'value' in val:
-                    init_vals[arg] = val['value']
-                    del kwargs2[arg]['value']
+                if 'val' in val:
+                    init_vals[arg] = val['val']
+                    del kwargs2[arg]['val']
 
                 if shape_by_conn or 'shape_by_conn' in val or 'copy_shape' in val:
-                    if val.get('shape') is not None or val.get('value') is not None:
-                        raise RuntimeError(f"{self.msginfo}: Can't set 'shape' or 'value' for "
+                    if val.get('shape') is not None or val.get('val') is not None:
+                        raise RuntimeError(f"{self.msginfo}: Can't set 'shape' or 'val' for "
                                            f"variable '{arg}' along with 'copy_shape' or "
                                            "'shape_by_conn'.")
 
@@ -413,18 +431,20 @@ class ExecComp(ExplicitComponent):
                     if kvalue is not None:
                         current_meta[kname] = kvalue
 
-                new_val = kwargs[var].get('value')
+                new_val = kwargs[var].get('val')
                 if new_val is not None:
-                    current_meta['value'] = new_val
+                    # val is normally ensured to be a numpy array in add_input/add_output,
+                    # do the same here...
+                    current_meta['val'] = np.atleast_1d(new_val)
             else:
                 # new input and/or output.
                 if var in outs:
-                    dct = self.add_output(var, val, **meta)
+                    current_meta = self.add_output(var, val, **meta)
                 else:
-                    dct = self.add_input(var, val, **meta)
+                    current_meta = self.add_input(var, val, **meta)
 
             if var not in init_vals:
-                init_vals[var] = dct['value']
+                init_vals[var] = current_meta['val']
 
         self._codes = self._compile_exprs(self._exprs)
 
@@ -441,7 +461,7 @@ class ExecComp(ExplicitComponent):
         **kwargs : dict of named args
             Initial values of variables can be set by setting a named arg with the var name.  If
             the value is a dict it is assumed to contain metadata.  To set the initial value in
-            addition to other metadata, assign the initial value to the 'value' entry of the dict.
+            addition to other metadata, assign the initial value to the 'val' entry of the dict.
             Do not include for inputs whose default kwargs have been declared on previous
             expressions.
         """
@@ -581,9 +601,9 @@ class ExecComp(ExplicitComponent):
                 for out in sorted(outs):
                     for inp in ins:
                         if self.options['has_diag_partials']:
-                            ival = meta[inp]['value']
+                            ival = meta[inp]['val']
                             iarray = isinstance(ival, ndarray) and ival.size > 1
-                            oval = meta[out]['value']
+                            oval = meta[out]['val']
                             if iarray and isinstance(oval, ndarray) and oval.size > 1:
                                 if oval.size != ival.size:
                                     raise RuntimeError(
@@ -632,7 +652,7 @@ class ExecComp(ExplicitComponent):
         """
         for i, expr in enumerate(self._codes):
             try:
-                exec(expr, _expr_dict, _IODict(outputs, inputs))
+                exec(expr, _expr_dict, _IODict(outputs, inputs))  # nosec: limited to _expr_dict
             except Exception as err:
                 raise RuntimeError(f"{self.msginfo}: Error occurred evaluating '{self._exprs[i]}':"
                                    f"\n{err}")
@@ -645,7 +665,7 @@ class ExecComp(ExplicitComponent):
         ----------
         jac : Jacobian or None
             Ignored.
-        sub_do_ln : boolean
+        sub_do_ln : bool
             Flag indicating if the children should call linearize on their linear solvers.
         """
         if self._requires_fd:
@@ -672,8 +692,7 @@ class ExecComp(ExplicitComponent):
         Parameters
         ----------
         inputs : `VecWrapper`
-            `VecWrapper` containing parameters. (p)
-
+            `VecWrapper` containing parameters (p).
         partials : `Jacobian`
             Contains sub-jacobians.
         """
