@@ -1,6 +1,6 @@
 """MetaModel provides basic meta modeling capability."""
 from copy import deepcopy
-from itertools import chain, product
+from itertools import chain
 
 import numpy as np
 
@@ -8,20 +8,26 @@ from openmdao.core.explicitcomponent import ExplicitComponent
 from openmdao.surrogate_models.surrogate_model import SurrogateModel
 from openmdao.utils.class_util import overrides_method
 from openmdao.utils.name_maps import rel_key2abs_key
-from openmdao.warnings import issue_warning, DerivativesWarning
+from openmdao.utils.om_warnings import issue_warning, DerivativesWarning
+from openmdao.utils.array_utils import shape_to_len
 
 
 class MetaModelUnStructuredComp(ExplicitComponent):
-    """
+    r"""
     Class that creates a reduced order model for outputs from inputs.
 
     Each output may have its own surrogate model.
-    Training inputs and outputs are automatically created with 'train:' prepended to the
+    Training inputs and outputs are automatically created with 'train\_' prepended to the
     corresponding input/output name.
 
     For a Float variable, the training data is an array of length m,
     where m is the number of training points.
 
+    Parameters
+    ----------
+    **kwargs : dict of keyword arguments
+        Keyword arguments that will be mapped into the Component options.
+
     Attributes
     ----------
     train : bool
@@ -47,11 +53,6 @@ class MetaModelUnStructuredComp(ExplicitComponent):
     def __init__(self, **kwargs):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        **kwargs : dict of keyword arguments
-            Keyword arguments that will be mapped into the Component options.
         """
         super().__init__(**kwargs)
 
@@ -116,19 +117,19 @@ class MetaModelUnStructuredComp(ExplicitComponent):
 
         Parameters
         ----------
-        name : string
+        name : str
             Name of the input.
         val : float or ndarray
             Initial value for the input.
         training_data : float or ndarray
-            training data for this variable. Optional, can be set by the problem later.
+            Training data for this variable. Optional, can be set by the problem later.
         **kwargs : dict
             Additional agruments for add_input.
 
         Returns
         -------
         dict
-            metadata for added variable
+            Metadata for added variable.
         """
         metadata = super().add_input(name, val, **kwargs)
         vec_size = self.options['vec_size']
@@ -137,9 +138,9 @@ class MetaModelUnStructuredComp(ExplicitComponent):
             if metadata['shape'][0] != vec_size:
                 raise RuntimeError(f"{self.msginfo}: First dimension of input '{name}' "
                                    f"must be {vec_size}")
-            input_size = metadata['value'][0].size
+            input_size = metadata['val'][0].size
         else:
-            input_size = metadata['value'].size
+            input_size = metadata['val'].size
 
         if self._static_mode:
             surrogate_input_names = self._static_surrogate_input_names
@@ -149,10 +150,15 @@ class MetaModelUnStructuredComp(ExplicitComponent):
             self._input_size += input_size
         surrogate_input_names.append((name, input_size))
 
-        train_name = 'train:%s' % name
-        self.options.declare(train_name, default=None, desc='Training data for %s' % name)
+        train_name = f'train:{name}'
+        good_name = f'train_{name}'
+        self.options.declare(train_name, default=None, desc='Training data for %s' % name,
+                             deprecation=(f"The option '{train_name}' has been deprecated because "
+                                          f"it's not a valid python name.  Use '{good_name}' "
+                                          "instead.", good_name))
+        self.options.declare(good_name, default=None, desc='Training data for %s' % name)
         if training_data is not None:
-            self.options[train_name] = training_data
+            self.options[good_name] = training_data
 
         return metadata
 
@@ -162,7 +168,7 @@ class MetaModelUnStructuredComp(ExplicitComponent):
 
         Parameters
         ----------
-        name : string
+        name : str
             Name of the variable output.
         val : float or ndarray
             Initial value for the output. While the value is overwritten during execution, it is
@@ -177,7 +183,7 @@ class MetaModelUnStructuredComp(ExplicitComponent):
         Returns
         -------
         dict
-            metadata for added variable
+            Metadata for added variable.
         """
         metadata = super().add_output(name, val, **kwargs)
         vec_size = self.options['vec_size']
@@ -186,9 +192,7 @@ class MetaModelUnStructuredComp(ExplicitComponent):
             if metadata['shape'][0] != vec_size:
                 raise RuntimeError(f"{self.msginfo}: First dimension of output '{name}' "
                                    f"must be {vec_size}")
-            output_shape = metadata['shape'][1:]
-            if len(output_shape) == 0:
-                output_shape = 1
+            output_shape = tuple(metadata['shape'][1:])
         else:
             output_shape = metadata['shape']
 
@@ -209,10 +213,16 @@ class MetaModelUnStructuredComp(ExplicitComponent):
         else:
             metadata['default_surrogate'] = True
 
-        train_name = 'train:%s' % name
-        self.options.declare(train_name, default=None, desc='Training data for %s' % name)
+        train_name = f'train:{name}'
+        good_name = f'train_{name}'
+        self.options.declare(train_name, default=None, desc='Training data for %s' % name,
+                             deprecation=(f"The option '{train_name}' has been deprecated because "
+                                          f"it's not a valid python name.  Use '{good_name}' "
+                                          "instead.", good_name))
+        self.options.declare(good_name, default=None, desc='Training data for %s' % name)
+
         if training_data is not None:
-            self.options[train_name] = training_data
+            self.options[good_name] = training_data
 
         return metadata
 
@@ -252,7 +262,7 @@ class MetaModelUnStructuredComp(ExplicitComponent):
             # Sparse specification of partials for vectorized models.
             for wrt, n_wrt in self._surrogate_input_names:
                 for of, shape_of in self._surrogate_output_names:
-                    n_of = np.prod(shape_of)
+                    n_of = shape_to_len(shape_of)
                     rows = np.repeat(np.arange(n_of), n_wrt)
                     cols = np.tile(np.arange(n_wrt), n_of)
                     repeat = np.repeat(vec_arange, len(rows))
@@ -267,7 +277,7 @@ class MetaModelUnStructuredComp(ExplicitComponent):
                     self._declare_partials(of=of, wrt=wrt, dct=dct)
         else:
             dct = {
-                'value': None,
+                'val': None,
                 'dependent': True,
             }
             # Dense specification of partials for non-vectorized models.
@@ -342,9 +352,9 @@ class MetaModelUnStructuredComp(ExplicitComponent):
         Parameters
         ----------
         inputs : Vector
-            unscaled, dimensional input variables read via inputs[key]
+            Unscaled, dimensional input variables read via inputs[key].
         outputs : Vector
-            unscaled, dimensional output variables read via outputs[key]
+            Unscaled, dimensional output variables read via outputs[key].
         """
         vec_size = self.options['vec_size']
 
@@ -492,13 +502,17 @@ class MetaModelUnStructuredComp(ExplicitComponent):
         step : float
             Step size for approximation. Defaults to None, in which case the approximation
             method provides its default value.
-        form : string
+        form : str
             Form for finite difference, can be 'forward', 'backward', or 'central'. Defaults
             to None, in which case the approximation method provides its default value.
-        step_calc : string
-            Step type for finite difference, can be 'abs' for absolute', or 'rel' for
-            relative. Defaults to None, in which case the approximation method provides
-            its default value.
+        step_calc : str
+            Step type for computing the size of the finite difference step. It can be 'abs' for
+            absolute, 'rel_avg' for a size relative to the absolute value of the vector input, or
+            'rel_element' for a size relative to each value in the vector input. In addition, it
+            can be 'rel_legacy' for a size relative to the norm of the vector.  For backwards
+            compatibilty, it can be 'rel', which currently defaults to 'rel_legacy', but in the
+            future will default to 'rel_avg'. Defaults to None, in which case the approximation
+            method provides its default value.
         """
         if method == 'cs':
             raise ValueError('Complex step has not been tested for MetaModelUnStructuredComp')
@@ -512,9 +526,9 @@ class MetaModelUnStructuredComp(ExplicitComponent):
         Parameters
         ----------
         inputs : Vector
-            unscaled, dimensional input variables read via inputs[key]
+            Unscaled, dimensional input variables read via inputs[key].
         partials : Jacobian
-            sub-jac components written to partials[output_name, input_name]
+            Sub-jac components written to partials[output_name, input_name].
         """
         vec_size = self.options['vec_size']
 
@@ -526,7 +540,7 @@ class MetaModelUnStructuredComp(ExplicitComponent):
         for out_name, out_shape in self._surrogate_output_names:
             surrogate = self._metadata(out_name).get('surrogate')
             if vec_size > 1:
-                out_size = np.prod(out_shape)
+                out_size = shape_to_len(out_shape)
                 for j in range(vec_size):
                     flat_input = flat_inputs[j]
                     if overrides_method('linearize', surrogate, SurrogateModel):
@@ -554,7 +568,7 @@ class MetaModelUnStructuredComp(ExplicitComponent):
         missing_training_data = []
         num_sample = None
         for name, _ in chain(self._surrogate_input_names, self._surrogate_output_names):
-            train_name = 'train:' + name
+            train_name = f'train_{name}'
             val = self.options[train_name]
             if val is None:
                 missing_training_data.append(train_name)
@@ -577,7 +591,7 @@ class MetaModelUnStructuredComp(ExplicitComponent):
         # Assemble input data.
         idx = 0
         for name, sz in self._surrogate_input_names:
-            val = self.options['train:' + name]
+            val = self.options[f'train_{name}']
             if isinstance(val[0], float):
                 inputs[:, idx] = val
                 idx += 1
@@ -585,15 +599,16 @@ class MetaModelUnStructuredComp(ExplicitComponent):
                 for row_idx, v in enumerate(val):
                     v = np.asarray(v)
                     inputs[row_idx, idx:idx + sz] = v.flat
+                idx += sz
 
         # Assemble output data and train each output.
         for name, shape in self._surrogate_output_names:
-            output_size = np.prod(shape)
+            output_size = shape_to_len(shape)
 
             outputs = np.zeros((num_sample, output_size))
             self._training_output[name] = outputs
 
-            val = self.options['train:' + name]
+            val = self.options[f'train_{name}']
 
             if isinstance(val[0], float):
                 outputs[:, 0] = val
