@@ -1,5 +1,5 @@
 """Define a base class for all Drivers in OpenMDAO."""
-from collections import OrderedDict
+from itertools import chain
 import pprint
 import sys
 import os
@@ -12,40 +12,27 @@ from openmdao.core.constants import INT_DTYPE
 from openmdao.recorders.recording_manager import RecordingManager
 from openmdao.recorders.recording_iteration_stack import Recording
 from openmdao.utils.record_util import create_local_meta, check_path
-from openmdao.utils.general_utils import simple_warning, warn_deprecation, _prom2ivc_src_dict, \
-    _prom2ivc_src_name_iter
+from openmdao.utils.general_utils import _src_name_iter
 from openmdao.utils.mpi import MPI
 from openmdao.utils.options_dictionary import OptionsDictionary
 import openmdao.utils.coloring as coloring_mod
-from openmdao.utils.array_utils import sizes2offsets, convert_neg
+from openmdao.utils.array_utils import sizes2offsets
 from openmdao.vectors.vector import _full_slice
-
-
-def _check_debug_print_opts_valid(name, opts):
-    """
-    Check validity of debug_print option for Driver.
-
-    Parameters
-    ----------
-    name : str
-        The name of the option.
-    opts : list
-        The value of the debug_print option set by the user.
-    """
-    if not isinstance(opts, list):
-        raise ValueError("Option '%s' with value %s is not a list." % (name, opts))
-
-    _valid_opts = ['desvars', 'nl_cons', 'ln_cons', 'objs', 'totals']
-    for opt in opts:
-        if opt not in _valid_opts:
-            raise ValueError("Option '%s' contains value '%s' which is not one of %s." %
-                             (name, opt, _valid_opts))
+from openmdao.utils.indexer import indexer
+from openmdao.utils.om_warnings import issue_warning, DerivativesWarning, MPIWarning, \
+    warn_deprecation
+from openmdao.utils.hooks import _setup_hooks
 
 
 class Driver(object):
     """
     Top-level container for the systems and drivers.
 
+    Parameters
+    ----------
+    **kwargs : dict of keyword arguments
+        Keyword arguments that will be mapped into the Driver options.
+
     Attributes
     ----------
     fail : bool
@@ -91,8 +78,9 @@ class Driver(object):
         Metadata pertaining to total coloring.
     _total_jac_sparsity : dict, str, or None
         Specifies sparsity of sub-jacobians of the total jacobian. Only used by pyOptSparseDriver.
-    _res_jacs : dict
+    _res_subjacs : dict
         Dict of sparse subjacobians for use with certain optimizers, e.g. pyOptSparseDriver.
+        Keyed by sources and aliases.
     _total_jac : _TotalJacInfo or None
         Cached total jacobian handling object.
     """
@@ -100,11 +88,6 @@ class Driver(object):
     def __init__(self, **kwargs):
         """
         Initialize the driver.
-
-        Parameters
-        ----------
-        **kwargs : dict of keyword arguments
-            Keyword arguments that will be mapped into the Driver options.
         """
         self._rec_mgr = RecordingManager()
 
@@ -118,10 +101,10 @@ class Driver(object):
         # Driver options
         self.options = OptionsDictionary(parent_name=type(self).__name__)
 
-        self.options.declare('debug_print', types=list, check_valid=_check_debug_print_opts_valid,
+        self.options.declare('debug_print', types=list,
+                             values=['desvars', 'nl_cons', 'ln_cons', 'objs', 'totals'],
                              desc="List of what type of Driver variables to print at each "
-                                  "iteration. Valid items in list are 'desvars', 'ln_cons', "
-                                  "'nl_cons', 'objs', 'totals'",
+                                  "iteration.",
                              default=[])
 
         # Case recording options
@@ -129,12 +112,10 @@ class Driver(object):
 
         self.recording_options.declare('record_model_metadata', types=bool, default=True,
                                        desc='Deprecated. Recording of model metadata will always '
-                                       'be done',
+                                            'be done',
                                        deprecation="The recording option, record_model_metadata, "
-                                       "on Driver is "
-                                       "deprecated. Recording of model metadata will always "
-                                       "be done",
-                                       )
+                                                   "on Driver is deprecated. Recording of model "
+                                                   "metadata will always be done")
         self.recording_options.declare('record_desvars', types=bool, default=True,
                                        desc='Set to True to record design variables at the '
                                             'driver level')
@@ -149,7 +130,7 @@ class Driver(object):
                                             'driver level')
         self.recording_options.declare('includes', types=list, default=[],
                                        desc='Patterns for variables to include in recording. '
-                                       'Uses fnmatch wildcards')
+                                            'Uses fnmatch wildcards')
         self.recording_options.declare('excludes', types=list, default=[],
                                        desc='Patterns for vars to exclude in recording '
                                             '(processed post-includes). Uses fnmatch wildcards')
@@ -185,7 +166,7 @@ class Driver(object):
         self._coloring_info = coloring_mod._get_coloring_meta()
 
         self._total_jac_sparsity = None
-        self._res_jacs = {}
+        self._res_subjacs = {}
         self._total_jac = None
 
         self.fail = False
@@ -193,13 +174,13 @@ class Driver(object):
         self._declare_options()
         self.options.update(kwargs)
 
+        # Want to allow the setting of hooks on Drivers
+        _setup_hooks(self)
+
     def _get_inst_id(self):
         if self._problem is None:
             return None
-        probid = self._problem()._get_inst_id()
-        if probid is None:
-            return "driver"
-        return f"{probid}.driver"
+        return f"{self._problem()._get_inst_id()}.driver"
 
     @property
     def msginfo(self):
@@ -278,16 +259,12 @@ class Driver(object):
 
         # Determine if any design variables are discrete.
         self._designvars_discrete = [name for name, meta in self._designvars.items()
-                                     if meta['ivc_source'] in model._discrete_outputs]
+                                     if meta['source'] in model._discrete_outputs]
         if not self.supports['integer_design_vars'] and len(self._designvars_discrete) > 0:
             msg = "Discrete design variables are not supported by this driver: "
             msg += '.'.join(self._designvars_discrete)
             raise RuntimeError(msg)
 
-        con_set = set()
-        obj_set = set()
-        dv_set = set()
-
         self._remote_dvs = remote_dv_dict = {}
         self._remote_cons = remote_con_dict = {}
         self._dist_driver_vars = dist_dict = {}
@@ -301,7 +278,7 @@ class Driver(object):
             for dv, meta in self._designvars.items():
 
                 # For Auto-ivcs, we need to check the distributed metadata on the target instead.
-                if meta['ivc_source'].startswith('_auto_ivc.'):
+                if meta['source'].startswith('_auto_ivc.'):
                     for abs_name in model._var_allprocs_prom2abs_list['input'][dv]:
                         if abs_name in discrete_in:
                             # Discrete vars aren't distributed.
@@ -321,16 +298,18 @@ class Driver(object):
 
         # Now determine if later we'll need to allgather cons, objs, or desvars.
         if model.comm.size > 1 and model._subsystems_allprocs:
-            src_design_vars = _prom2ivc_src_dict(self._designvars)
-            responses = _prom2ivc_src_dict(self._responses)
+            loc_vars = set(model._outputs._abs_iter())
+            # some of these lists could have duplicate src names if aliases are used. We'll
+            # fix that when we convert to sets after the allgather.
+            remote_dvs = [n for n in _src_name_iter(self._designvars) if n not in loc_vars]
+            remote_cons = [n for n in _src_name_iter(self._cons) if n not in loc_vars]
+            remote_objs = [n for n in _src_name_iter(self._objs) if n not in loc_vars]
 
-            local_out_vars = set(model._outputs._abs_iter())
-            remote_dvs = set(src_design_vars) - local_out_vars
-            remote_cons = set(_prom2ivc_src_name_iter(self._cons)) - local_out_vars
-            remote_objs = set(_prom2ivc_src_name_iter(self._objs)) - local_out_vars
+            con_set = set()
+            obj_set = set()
+            dv_set = set()
 
-            all_remote_vois = model.comm.allgather(
-                (remote_dvs, remote_cons, remote_objs))
+            all_remote_vois = model.comm.allgather((remote_dvs, remote_cons, remote_objs))
             for rem_dvs, rem_cons, rem_objs in all_remote_vois:
                 con_set.update(rem_cons)
                 obj_set.update(rem_objs)
@@ -339,52 +318,57 @@ class Driver(object):
             # If we have remote VOIs, pick an owning rank for each and use that
             # to bcast to others later
             owning_ranks = model._owning_rank
-            sizes = model._var_sizes['nonlinear']['output']
+            abs2idx = model._var_allprocs_abs2idx
+            abs2meta_out = model._var_allprocs_abs2meta['output']
+            sizes = model._var_sizes['output']
             rank = model.comm.rank
             nprocs = model.comm.size
-            for i, (vname, meta) in enumerate(model._var_allprocs_abs2meta['output'].items()):
-                if vname in responses:
-                    indices = responses[vname].get('indices')
-                elif vname in src_design_vars:
-                    indices = src_design_vars[vname].get('indices')
-                else:
-                    continue
+
+            # Loop over all VOIs.
+            for vname, voimeta in chain(self._responses.items(), self._designvars.items()):
+                # vname may be a source or an alias
+
+                indices = voimeta['indices']
+                vsrc = voimeta['source']
+
+                meta = abs2meta_out[vsrc]
+                i = abs2idx[vsrc]
 
                 if meta['distributed']:
 
-                    idx = model._var_allprocs_abs2idx['nonlinear'][vname]
-                    dist_sizes = model._var_sizes['nonlinear']['output'][:, idx]
-                    total_dist_size = np.sum(dist_sizes)
+                    dist_sizes = sizes[:, i]
+                    tot_size = np.sum(dist_sizes)
 
                     # Determine which indices are on our proc.
                     offsets = sizes2offsets(dist_sizes)
 
                     if indices is not None:
-                        indices = convert_neg(indices, total_dist_size)
+                        indices = indices.shaped_array()
                         true_sizes = np.zeros(nprocs, dtype=INT_DTYPE)
                         for irank in range(nprocs):
                             dist_inds = indices[np.logical_and(indices >= offsets[irank],
                                                                indices < (offsets[irank] +
                                                                           dist_sizes[irank]))]
+                            true_sizes[irank] = dist_inds.size
                             if irank == rank:
                                 local_indices = dist_inds - offsets[rank]
                                 distrib_indices = dist_inds
 
-                            true_sizes[irank] = dist_inds.size
-                        dist_dict[vname] = (local_indices, true_sizes, distrib_indices)
+                        ind = indexer(local_indices, src_shape=(tot_size,), flat_src=True)
+                        dist_dict[vname] = (ind, true_sizes, distrib_indices)
                     else:
                         dist_dict[vname] = (_full_slice, dist_sizes,
                                             slice(offsets[rank], offsets[rank] + dist_sizes[rank]))
 
                 else:
-                    owner = owning_ranks[vname]
+                    owner = owning_ranks[vsrc]
                     sz = sizes[owner, i]
 
-                    if vname in dv_set:
+                    if vsrc in dv_set:
                         remote_dv_dict[vname] = (owner, sz)
-                    if vname in con_set:
+                    if vsrc in con_set:
                         remote_con_dict[vname] = (owner, sz)
-                    if vname in obj_set:
+                    if vsrc in obj_set:
                         remote_obj_dict[vname] = (owner, sz)
 
         self._remote_responses = self._remote_cons.copy()
@@ -445,7 +429,7 @@ class Driver(object):
         myinputs = myoutputs = myresiduals = []
 
         if recording_options['record_outputs']:
-            myoutputs = sorted([n for n, prom in abs2prom.items() if check_path(prom, incl, excl)])
+            myoutputs = [n for n, prom in abs2prom.items() if check_path(prom, incl, excl)]
 
             model_outs = model._outputs
 
@@ -462,23 +446,23 @@ class Driver(object):
 
         myoutputs = set(myoutputs)
         if recording_options['record_desvars']:
-            myoutputs.update(self._designvars)
+            myoutputs.update(_src_name_iter(self._designvars))
         if recording_options['record_objectives'] or recording_options['record_responses']:
-            myoutputs.update(self._objs)
+            myoutputs.update(_src_name_iter(self._objs))
         if recording_options['record_constraints'] or recording_options['record_responses']:
-            myoutputs.update(self._cons)
+            myoutputs.update(_src_name_iter(self._cons))
 
         # inputs (if in options). inputs use _absolute_ names for includes/excludes
         if 'record_inputs' in recording_options:
             if recording_options['record_inputs']:
-                # sort the results since _var_allprocs_abs2prom isn't ordered
-                myinputs = sorted([n for n in model._var_allprocs_abs2prom['input']
-                                  if check_path(n, incl, excl)])
+                myinputs = [n for n in model._var_allprocs_abs2prom['input']
+                            if check_path(n, incl, excl)]
 
+        # sort lists to ensure that vars are iterated over in the same order on all procs
         vars2record = {
-            'input': myinputs,
-            'output': list(myoutputs),
-            'residual': myresiduals
+            'input': sorted(myinputs),
+            'output': sorted(myoutputs),
+            'residual': sorted(myresiduals)
         }
 
         return vars2record
@@ -488,8 +472,7 @@ class Driver(object):
         Set up case recording.
         """
         self._filtered_vars_to_record = self._get_vars_to_record(self.recording_options)
-
-        self._rec_mgr.startup(self)
+        self._rec_mgr.startup(self, self._problem().comm)
 
     def _get_voi_val(self, name, meta, remote_vois, driver_scaling=True,
                      get_remote=True, rank=None):
@@ -528,36 +511,35 @@ class Driver(object):
         model = self._problem().model
         comm = model.comm
         get = model._outputs._abs_get_val
-        distributed_vars = self._dist_driver_vars
         indices = meta['indices']
 
-        if meta.get('ivc_source') is not None:
-            src_name = meta['ivc_source']
-        else:
-            src_name = name
+        src_name = meta['source']
+
+        # If there's an alias, use that for driver related stuff
+        drv_name = name if meta.get('alias') else src_name
 
         if MPI:
-            distributed = comm.size > 0 and src_name in distributed_vars
+            distributed = comm.size > 0 and drv_name in self._dist_driver_vars
         else:
             distributed = False
 
-        if src_name in remote_vois:
-            owner, size = remote_vois[src_name]
+        if drv_name in remote_vois:
+            owner, size = remote_vois[drv_name]
             # if var is distributed or only gathering to one rank
             # TODO - support distributed var under a parallel group.
             if owner is None or rank is not None:
                 val = model.get_val(src_name, get_remote=get_remote, rank=rank, flat=True)
                 if indices is not None:
-                    val = val[indices]
+                    val = val[indices.flat()]
             else:
                 if owner == comm.rank:
                     if indices is None:
-                        val = get(name).copy()
+                        val = get(src_name, flat=True).copy()
                     else:
-                        val = get(name)[indices]
+                        val = get(src_name, flat=True)[indices.as_array()]
                 else:
                     if indices is not None:
-                        size = len(indices)
+                        size = indices.indexed_src_size
                     val = np.empty(size)
 
                 if get_remote:
@@ -565,13 +547,12 @@ class Driver(object):
 
         elif distributed:
             local_val = model.get_val(src_name, get_remote=False, flat=True)
-            local_indices, sizes, _ = distributed_vars[src_name]
-            if local_indices is not None:
-                local_val = local_val[local_indices]
+            local_indices, sizes, _ = self._dist_driver_vars[drv_name]
+            if local_indices is not _full_slice:
+                local_val = local_val[local_indices()]
 
             if get_remote:
-                if not local_val.flags['C_CONTIGUOUS']:
-                    local_val = np.ascontiguousarray(local_val)
+                local_val = np.ascontiguousarray(local_val)
                 offsets = np.zeros(sizes.size, dtype=INT_DTYPE)
                 offsets[1:] = np.cumsum(sizes[:-1])
                 val = np.zeros(np.sum(sizes))
@@ -580,24 +561,26 @@ class Driver(object):
                 val = local_val
 
         else:
-            if name in self._designvars_discrete:
+            if src_name in model._discrete_outputs:
                 val = model._discrete_outputs[src_name]
-
-                # At present, only integers are supported by OpenMDAO drivers.
-                # We check the values here.
-                msg = "Only integer scalars or ndarrays are supported as values for " + \
-                      "discrete variables when used as a design variable. "
-                if np.isscalar(val) and not isinstance(val, (int, np.integer)):
-                    msg += "A value of type '{}' was specified.".format(val.__class__.__name__)
-                    raise ValueError(msg)
-                elif isinstance(val, np.ndarray) and not np.issubdtype(val[0], np.integer):
-                    msg += "An array of type '{}' was specified.".format(val[0].__class__.__name__)
-                    raise ValueError(msg)
-
+                if name in self._designvars_discrete:
+                    # At present, only integers are supported by OpenMDAO drivers.
+                    # We check the values here.
+                    if not ((np.isscalar(val) and isinstance(val, (int, np.integer))) or
+                            (isinstance(val, np.ndarray) and np.issubdtype(val[0], np.integer))):
+                        if np.isscalar(val):
+                            suffix = f"A value of type '{type(val).__name__}' was specified."
+                        elif isinstance(val, np.ndarray):
+                            suffix = f"An array of type '{val.dtype.name}' was specified."
+                        else:
+                            suffix = ''
+                        raise ValueError("Only integer scalars or ndarrays are supported as values "
+                                         "for discrete variables when used as a design variable. "
+                                         + suffix)
             elif indices is None:
-                val = get(src_name).copy()
+                val = get(src_name, flat=True).copy()
             else:
-                val = get(src_name)[indices]
+                val = get(src_name, flat=True)[indices.as_array()]
 
         if self._has_scaling and driver_scaling:
             # Scale design variable values
@@ -611,7 +594,7 @@ class Driver(object):
 
         return val
 
-    def get_design_var_values(self, get_remote=True):
+    def get_design_var_values(self, get_remote=True, driver_scaling=True):
         """
         Return the design variable values.
 
@@ -623,19 +606,26 @@ class Driver(object):
             in the Problem's MPI communicator.
             If False, only retrieve the value if it is on the current process, or only the part
             of the value that's on the current process for a distributed variable.
+        driver_scaling : bool
+            When True, return values that are scaled according to either the adder and scaler or
+            the ref and ref0 values that were specified when add_design_var, add_objective, and
+            add_constraint were called on the model. Default is True.
 
         Returns
         -------
         dict
            Dictionary containing values of each design variable.
         """
-        return {n: self._get_voi_val(n, dv, self._remote_dvs, get_remote=get_remote)
+        return {n: self._get_voi_val(n, dv, self._remote_dvs, get_remote=get_remote,
+                                     driver_scaling=driver_scaling)
                 for n, dv in self._designvars.items()}
 
     def set_design_var(self, name, value, set_remote=True):
         """
         Set the value of a design variable.
 
+        'name' can be a promoted output name or an alias.
+
         Parameters
         ----------
         name : str
@@ -649,17 +639,13 @@ class Driver(object):
         problem = self._problem()
         meta = self._designvars[name]
 
-        src_name = meta['ivc_source']
+        src_name = meta['source']
 
         # if the value is not local, don't set the value
         if (src_name in self._remote_dvs and
                 problem.model._owning_rank[src_name] != problem.comm.rank):
             return
 
-        indices = meta['indices']
-        if indices is None:
-            indices = _full_slice
-
         if name in self._designvars_discrete:
 
             # Note, drivers set values here and generally should know it is setting an integer.
@@ -678,10 +664,14 @@ class Driver(object):
 
         elif problem.model._outputs._contains_abs(src_name):
             desvar = problem.model._outputs._abs_get_val(src_name)
-            if src_name in self._dist_driver_vars:
-                loc_idxs, _, dist_idxs = self._dist_driver_vars[src_name]
+            if name in self._dist_driver_vars:
+                loc_idxs, _, dist_idxs = self._dist_driver_vars[name]
             else:
-                loc_idxs = indices
+                loc_idxs = meta['indices']
+                if loc_idxs is None:
+                    loc_idxs = _full_slice
+                else:
+                    loc_idxs = loc_idxs()
                 dist_idxs = _full_slice
 
             if set_remote:
@@ -727,10 +717,10 @@ class Driver(object):
 
         Parameters
         ----------
-        ctype : string
+        ctype : str
             Default is 'all'. Optionally return just the inequality constraints
             with 'ineq' or the equality constraints with 'eq'.
-        lintype : string
+        lintype : str
             Default is 'all'. Optionally return just the linear constraints
             with 'linear' or the nonlinear constraints with 'nonlinear'.
         driver_scaling : bool
@@ -795,8 +785,8 @@ class Driver(object):
         int
             Total size of design vars.
         """
-        self._objs = objs = OrderedDict()
-        self._cons = cons = OrderedDict()
+        self._objs = objs = {}
+        self._cons = cons = {}
 
         model._setup_driver_units()
 
@@ -824,7 +814,7 @@ class Driver(object):
 
         Returns
         -------
-        boolean
+        bool
             Failure flag; True if failed to converge, False is successful.
         """
         with RecordingDebugging(self._get_name(), self.iter_count, self):
@@ -846,13 +836,13 @@ class Driver(object):
 
         Parameters
         ----------
-        of : list of variable name strings or None
+        of : list of variable name str or None
             Variables whose derivatives will be computed. Default is None, which
             uses the driver's objectives and constraints.
-        wrt : list of variable name strings or None
+        wrt : list of variable name str or None
             Variables with respect to which the derivatives will be computed.
             Default is None, which uses the driver's desvars.
-        return_format : string
+        return_format : str
             Format to return the derivatives. Default is a 'flat_dict', which
             returns them in a dictionary whose keys are tuples of form (of, wrt). For
             the scipy optimizer, 'array' is also supported.
@@ -925,7 +915,13 @@ class Driver(object):
         """
         Record an iteration of the current Driver.
         """
-        record_iteration(self, self._problem(), self._get_name())
+        if self._problem:
+            record_iteration(self, self._problem(), self._get_name())
+        else:
+            raise RuntimeError(f'{self.msginfo} attempted to record iteration but '
+                               'driver has not been initialized; `run_model()`, '
+                               '`run_driver()`, or `final_setup()` must be called '
+                               'before recording.')
 
     def _get_recorder_metadata(self, case_name):
         """
@@ -1004,7 +1000,6 @@ class Driver(object):
         coloring : str
             A coloring filename.  If no arg is passed, filename will be determined
             automatically.
-
         """
         if self.supports['simultaneous_derivatives']:
             if coloring_mod._force_dyn_coloring and coloring is coloring_mod._STD_COLORING_FNAME:
@@ -1081,7 +1076,8 @@ class Driver(object):
 
         problem = self._problem()
         if not problem.model._use_derivatives:
-            simple_warning("Derivatives are turned off.  Skipping simul deriv coloring.")
+            issue_warning("Derivatives are turned off.  Skipping simul deriv coloring.",
+                          category=DerivativesWarning)
             return
 
         total_coloring = self._get_static_coloring()
@@ -1121,6 +1117,17 @@ class Driver(object):
             Data used to create html file.
         """
         from openmdao.visualization.scaling_viewer.scaling_report import view_driver_scaling
+
+        # Run the model if it hasn't been run yet.
+        try:
+            prob = self._problem()
+        except TypeError:
+            raise RuntimeError("Either 'run_model' or 'final_setup' must be called before the "
+                               "scaling report can be generated.")
+
+        if prob._run_counter < 0:
+            prob.run_model()
+
         return view_driver_scaling(self, outfile=outfile, show_browser=show_browser, jac=jac,
                                    title=title)
 
@@ -1197,6 +1204,15 @@ class RecordingDebugging(Recording):
 
     Handles doing the case recording and also the Driver
     debugging printing.
+
+    Parameters
+    ----------
+    name : str
+        Name of object getting recorded.
+    iter_count : int
+        Current counter of iterations completed.
+    recording_requester : object
+        Object to which this recorder is attached.
     """
 
     def __enter__(self):
