@@ -17,6 +17,7 @@ from openmdao.utils.testing_utils import use_tempdirs
 from openmdao.test_suite.components.paraboloid import Paraboloid
 from openmdao.test_suite.components.sellar import SellarDerivatives
 from openmdao.test_suite.components.simple_comps import DoubleArrayComp, NonSquareArrayComp
+from openmdao.utils.om_warnings import OMDeprecationWarning
 
 from openmdao.utils.mpi import MPI
 
@@ -155,7 +156,7 @@ class TestDriver(unittest.TestCase):
         prob.setup()
         prob.run_driver()
 
-        with assert_warning(DeprecationWarning, "'global_names' is deprecated in calls to _compute_totals. Use 'use_abs_names' instead."):
+        with assert_warning(OMDeprecationWarning, "'global_names' is deprecated in calls to _compute_totals. Use 'use_abs_names' instead."):
             derivs = prob.driver._compute_totals(of=['comp.y1'], wrt=['px.x'], global_names=True,
                                                  return_format='dict')
 
@@ -287,7 +288,41 @@ class TestDriver(unittest.TestCase):
         with self.assertRaises(ValueError) as context:
             prob.driver.options['debug_print'] = ['bad_option']
         self.assertEqual(str(context.exception),
-                         "Option 'debug_print' contains value 'bad_option' which is not one of ['desvars', 'nl_cons', 'ln_cons', 'objs', 'totals'].")
+                         "Driver: Value (['bad_option']) of option 'debug_print' is not one of "
+                         "['desvars', 'nl_cons', 'ln_cons', 'objs', 'totals'].")
+
+    def test_debug_print_approx(self):
+
+        prob = om.Problem()
+        prob.model = model = SellarDerivatives()
+
+        model.add_design_var('z')
+        model.add_objective('obj')
+        model.add_constraint('con1', lower=0)
+        model.add_constraint('con2', lower=0, linear=True)
+        prob.set_solver_print(level=0)
+
+        prob.driver = om.ScipyOptimizeDriver()
+        prob.driver.options['optimizer'] = 'SLSQP'
+        prob.driver.options['tol'] = 1e-9
+        prob.driver.options['disp'] = False
+        prob.driver.options['maxiter'] = 0
+
+        prob.model.approx_totals()
+        prob.driver.options['debug_print'] = ['totals']
+
+        prob.setup()
+
+        stdout = sys.stdout
+        strout = StringIO()
+        sys.stdout = strout
+        try:
+            prob.run_driver()
+        finally:
+            sys.stdout = stdout
+        output = strout.getvalue().split('\n')
+        self.assertIn("{('obj_cmp.obj', 'z'):", output[12])
+        self.assertIn("{('con_cmp1.con1', 'z'):", output[13])
 
     def test_debug_print_desvar_physical_with_indices(self):
         prob = om.Problem()
@@ -450,7 +485,7 @@ class TestDriver(unittest.TestCase):
         prob = om.Problem()
         msg = "The recording option, record_model_metadata, on Driver is deprecated. " \
               "Recording of model metadata will always be done"
-        with assert_warning(DeprecationWarning, msg):
+        with assert_warning(OMDeprecationWarning, msg):
             prob.driver.recording_options['record_model_metadata'] = True
 
     def test_units_basic(self):
@@ -462,13 +497,13 @@ class TestDriver(unittest.TestCase):
 
         model.add_subsystem('p', ivc, promotes=['x'])
         model.add_subsystem('comp1', om.ExecComp('y1 = 2.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y1={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0, 'units': 'degF'},
+                                                 y1={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y1'])
 
         model.add_subsystem('comp2', om.ExecComp('y2 = 3.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y2={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0, 'units': 'degF'},
+                                                 y2={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y2'])
 
         model.add_design_var('x', units='degC', lower=0.0, upper=100.0)
@@ -521,13 +556,13 @@ class TestDriver(unittest.TestCase):
 
         model.add_subsystem('p', ivc, promotes=['x'])
         model.add_subsystem('comp1', om.ExecComp('y1 = 2.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y1={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0, 'units': 'degF'},
+                                                 y1={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y1'])
 
         model.add_subsystem('comp2', om.ExecComp('y2 = 3.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y2={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0, 'units': 'degF'},
+                                                 y2={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y2'])
 
         model.add_design_var('x', units='degF', lower=32.0, upper=212.0)
@@ -564,13 +599,13 @@ class TestDriver(unittest.TestCase):
 
         model.add_subsystem('p', ivc, promotes=['x'])
         model.add_subsystem('comp1', om.ExecComp('y1 = 2.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y1={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0, 'units': 'degF'},
+                                                 y1={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y1'])
 
         model.add_subsystem('comp2', om.ExecComp('y2 = 3.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y2={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0, 'units': 'degF'},
+                                                 y2={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y2'])
 
         model.add_design_var('x', units='degC', lower=0.0, upper=100.0, scaler=3.5, adder=77.0)
@@ -672,8 +707,8 @@ class TestDriver(unittest.TestCase):
 
         model.add_subsystem('p', ivc, promotes=['x'])
         model.add_subsystem('comp1', om.ExecComp('y1 = 2.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y1={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0, 'units': 'degF'},
+                                                 y1={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y1'])
 
         model.add_design_var('x', units='ft', lower=0.0, upper=100.0, scaler=3.5, adder=77.0)
@@ -693,8 +728,8 @@ class TestDriver(unittest.TestCase):
 
         model.add_subsystem('p', ivc, promotes=['x'])
         model.add_subsystem('comp1', om.ExecComp('y1 = 2.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y1={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0, 'units': 'degF'},
+                                                 y1={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y1'])
 
         model.add_constraint('x', units='ft', lower=0.0, upper=100.0)
@@ -714,8 +749,8 @@ class TestDriver(unittest.TestCase):
 
         model.add_subsystem('p', ivc, promotes=['x'])
         model.add_subsystem('comp1', om.ExecComp('y1 = 2.0*x',
-                                                 x={'value': 2.0},
-                                                 y1={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0},
+                                                 y1={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y1'])
 
         model.add_design_var('x', units='ft', lower=0.0, upper=100.0, scaler=3.5, adder=77.0)
@@ -735,8 +770,8 @@ class TestDriver(unittest.TestCase):
 
         model.add_subsystem('p', ivc, promotes=['x'])
         model.add_subsystem('comp1', om.ExecComp('y1 = 2.0*x',
-                                                 x={'value': 2.0},
-                                                 y1={'value': 2.0, 'units': 'degF'}),
+                                                 x={'val': 2.0},
+                                                 y1={'val': 2.0, 'units': 'degF'}),
                             promotes=['x', 'y1'])
 
         model.add_constraint('x', units='ft', lower=0.0, upper=100.0)
@@ -788,50 +823,6 @@ class TestDriver(unittest.TestCase):
         assert_near_equal(totals['sub.comp.f_xy', 'sub.y']['J_fd'], [[1.58e2]], 1e-5)
 
 
-class TestDriverFeature(unittest.TestCase):
-
-    def test_specify_units(self):
-        import openmdao.api as om
-
-        prob = om.Problem()
-        model = prob.model
-
-        model.add_subsystem('comp1', om.ExecComp('y1 = 2.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y1={'value': 2.0, 'units': 'degF'}),
-                            promotes=['x', 'y1'])
-
-        model.add_subsystem('comp2', om.ExecComp('y2 = 3.0*x',
-                                                 x={'value': 2.0, 'units': 'degF'},
-                                                 y2={'value': 2.0, 'units': 'degF'}),
-                            promotes=['x', 'y2'])
-
-        model.set_input_defaults('x', 35.0, units='degF')
-
-        model.add_design_var('x', units='degC', lower=0.0, upper=100.0)
-        model.add_constraint('y1', units='degC', lower=0.0, upper=100.0)
-        model.add_objective('y2', units='degC')
-
-        prob.setup()
-        prob.run_driver()
-
-        print('Model variables')
-        assert_near_equal(prob.get_val('x', indices=[0]), 35.0, 1e-8)
-        assert_near_equal(prob.get_val('comp2.y2', indices=[0]), 105.0, 1e-8)
-        assert_near_equal(prob.get_val('comp1.y1', indices=[0]), 70.0, 1e-8)
-
-        print('')
-        print('Driver variables')
-        dv = prob.driver.get_design_var_values()
-        assert_near_equal(dv['x'][0], 3.0 * 5 / 9, 1e-8)
-
-        obj = prob.driver.get_objective_values(driver_scaling=True)
-        assert_near_equal(obj['comp2.y2'][0], 73.0 * 5 / 9, 1e-8)
-
-        con = prob.driver.get_constraint_values(driver_scaling=True)
-        assert_near_equal(con['comp1.y1'][0], 38.0 * 5 / 9, 1e-8)
-
-
 @unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
 class TestDriverMPI(unittest.TestCase):
 
@@ -873,13 +864,11 @@ class TestDriverMPI(unittest.TestCase):
         class DistribComp(om.ExplicitComponent):
 
             def setup(self):
-                self.options['distributed'] = True
+                self.add_input('w', val=1., distributed=True) # this will connect to a non-distributed IVC
+                self.add_input('x', shape=size, distributed=True) # this will connect to a distributed IVC
 
-                self.add_input('w', val=1.) # this will connect to a non-distributed IVC
-                self.add_input('x', shape=size) # this will connect to a distributed IVC
-
-                self.add_output('y', shape=1) # all-gathered output, duplicated on all procs
-                self.add_output('z', shape=size) # distributed output
+                self.add_output('y', shape=1, distributed=True) # all-gathered output, duplicated on all procs
+                self.add_output('z', shape=size, distributed=True) # distributed output
 
                 self.declare_partials('y', 'x')
                 self.declare_partials('y', 'w')
