@@ -2,7 +2,7 @@ import unittest
 import numpy as np
 
 import openmdao.api as om
-from openmdao.utils.assert_utils import assert_near_equal, assert_warning
+from openmdao.utils.assert_utils import assert_near_equal, assert_warning, assert_check_totals
 from openmdao.utils.om_warnings import  OMDeprecationWarning
 
 class Inner(om.Group):
@@ -16,7 +16,7 @@ class Outer(om.Group):
         self.add_subsystem('inner', Inner())
     def configure(self):
         self.promotes('inner', inputs=[('comp.x', 'desvar_x')],
-                      src_indices=np.array([[0, 1], [0, 1], [0, 1]]), flat_src_indices=True, src_shape=2)
+                      src_indices=np.array([0, 1, 0, 1, 0, 1]), flat_src_indices=True, src_shape=2)
 
 
 class SrcIndicesTestCase(unittest.TestCase):
@@ -189,33 +189,32 @@ class SrcIndicesTestCase(unittest.TestCase):
         class Burn1(om.Group):
             def setup(self):
                 self.add_subsystem('comp1', om.ExecComp(['y1=x*2'], y1=np.ones(4), x=np.ones(4)),
-                                promotes_outputs=['*'])
+                                   promotes_outputs=['*'])
 
                 self.add_subsystem('comp2', om.ExecComp(['y2=x*3'], y2=np.ones(4), x=np.ones(4)),
-                                promotes_outputs=['*'])
+                                   promotes_outputs=['*'])
 
             def configure(self):
                 self.promotes('comp1', inputs=[('x', 'design:x')],
-                            src_indices=[0, 0, 0, 0], flat_src_indices=True)
+                              src_indices=[0, 0, 0, 0], flat_src_indices=True)
 
                 self.set_input_defaults('design:x', 75.3)
 
 
         class Traj(om.Group):
             def setup(self):
-                self.add_subsystem('burn1', Burn1(),
-                                promotes_outputs=['*'])
+                self.add_subsystem('burn1', Burn1(), promotes_outputs=['*'])
 
             def configure(self):
                 self.promotes('burn1', inputs=['design:x'],
-                            src_indices=[0, 0, 0, 0], flat_src_indices=True)
+                              src_indices=[0, 0, 0, 0], flat_src_indices=True)
 
         prob = om.Problem(model=Traj())
 
         prob.setup()
         prob.run_model()
 
-        assert_near_equal(prob['design:x'], [75.3]*4)
+        assert_near_equal(prob['design:x'], [75.3])
         assert_near_equal(prob['y1'], [75.3*2]*4)
         assert_near_equal(prob['y2'], [1*3]*4)
 
@@ -318,12 +317,12 @@ class SrcIndicesTestCase(unittest.TestCase):
         with self.assertRaises(RuntimeError) as cm:
             p.setup()
 
-        self.assertEqual(cm.exception.args[0], "'G.g1' <class Group>: Promoted src_shape of (3, 3) for 'x' in 'G.g1.C1' differs from src_shape (3, 2) for 'x' in 'G'.")
+        self.assertEqual(cm.exception.args[0], "In connection from '_auto_ivc.v0' to 'G.g1.C1.x', error was: Promoted src_shape of (3, 3) for 'G.g1.C1.x' differs from src_shape (3, 2) for 'x'.")
 
     def test_src_indices_on_promotes(self):
         src_shape = (3, 3)
         tgt_shape = (2, 2)
-        src_indices = [[4, 5], [7, 9]]
+        src_indices = [4, 5, 7, 9]
         flat_src_indices = True
 
         class MyComp(om.ExplicitComponent):
@@ -344,64 +343,53 @@ class SrcIndicesTestCase(unittest.TestCase):
                             flat_src_indices=flat_src_indices)
         p.model.set_input_defaults('x', src_shape=src_shape)
 
-        with self.assertRaises(RuntimeError) as cm:
+        with self.assertRaises(IndexError) as cm:
             p.setup()
 
         self.assertEqual(cm.exception.args[0],
-                         "'C1' <class MyComp>: When promoting 'C1.x' with src_indices [[4 5] [7 9]] and "
-                         "source shape (3, 3): index 9 is out of bounds for source dimension of size 9.")
-
-    def test_connect_src_indices_deprecated(self):
-        class MyComp(om.ExplicitComponent):
-            def setup(self):
-                self.add_input('x', np.ones(3))
+                         "When promoting 'x' from system 'C1' with src_indices [4 5 7 9] and src_shape (3, 3): "
+                         "index 9 is out of bounds for source dimension of size 9.")
 
+    def test_connect_slice_src_indices_not_full_size(self):
         p = om.Problem()
 
-        p.model.add_subsystem('indep', om.IndepVarComp('x', np.ones(5)))
-        p.model.add_subsystem('C1', MyComp())
+        ivc = p.model.add_subsystem('ivc', om.IndepVarComp())
+        M_np = np.arange(9)[::-1].reshape((3, 3))
+        ivc.add_output('M', val=M_np)
 
-        with assert_warning(OMDeprecationWarning, "<class Group>: When connecting "
-                            "from 'indep.x' to 'C1.x': 'src_indices=(1, 0, 2)' is "
-                            "specified in a deprecated format. In a future release, "
-                            "'src_indices' will be expected to use NumPy array indexing, "  
-                            "so replace the existing src_indices with [1, 0, 2]."):
-            p.model.connect('indep.x', 'C1.x', src_indices=(1, 0, 2))
+        exec = p.model.add_subsystem('exec', om.ExecComp())
 
-    def test_connect_src_indices_deprecated2(self):
-        class MyComp(om.ExplicitComponent):
-            def setup(self):
-                self.add_input('x', np.ones(3))
+        exec.add_expr('A = B', A={'shape': (3,)}, B={'shape': (3,)})
 
+        p.model.connect('ivc.M', 'exec.B', src_indices=om.slicer[:1])
+
+        p.setup()
+
+        p.run_model()
+
+        assert_near_equal(p.get_val('exec.B').ravel(), p.get_val('ivc.M')[:1].ravel())
+        assert_near_equal(p.get_val('exec.A').ravel(), p.get_val('ivc.M')[:1].ravel())
+
+    def test_promote_slice_src_indices_not_full_size(self):
         p = om.Problem()
 
-        p.model.add_subsystem('indep', om.IndepVarComp('x', np.ones((4,3))))
-        p.model.add_subsystem('C1', MyComp())
+        ivc = p.model.add_subsystem('ivc', om.IndepVarComp())
+        M_np = np.arange(9)[::-1].reshape((3, 3))
+        ivc.add_output('M', val=M_np)
 
-        with assert_warning(OMDeprecationWarning, "<class Group>: When connecting "
-                            "from 'indep.x' to 'C1.x': 'src_indices=((1, 2), (1, 1), (3, 1))' is "
-                            "specified in a deprecated format. In a future release, "
-                            "'src_indices' will be expected to use NumPy array indexing, "  
-                            "so replace the existing src_indices with ([1, 1, 3], [2, 1, 1])."):
-            p.model.connect('indep.x', 'C1.x', src_indices=((1,2), (1, 1), (3,1)))
+        exec = p.model.add_subsystem('exec', om.ExecComp())
 
-    def test_promotes_src_indices_deprecated(self):
-        class MyComp(om.ExplicitComponent):
-            def setup(self):
-                self.add_input('x', np.ones(3))
+        exec.add_expr('A = B', A={'shape': (3,)}, B={'shape': (3,)})
 
-        p = om.Problem()
+        p.model.promotes('ivc', outputs=['M'])
+        p.model.promotes('exec', inputs=[('B', 'M')], src_indices=om.slicer[:1], src_shape=(3, 3))
 
-        p.model.add_subsystem('indep', om.IndepVarComp('x', np.ones(5)), promotes=['*'])
-        p.model.add_subsystem('C1', MyComp())
+        p.setup()
 
-        with assert_warning(OMDeprecationWarning, "<class Group>: When promoting ['x'] "
-                            "from 'C1': 'src_indices=(1, 0, 2)' is "
-                            "specified in a deprecated format. In a future release, "
-                            "'src_indices' will be expected to use NumPy array indexing, "  
-                            "so replace the existing src_indices with [1, 0, 2]."):
-            p.model.promotes('C1', inputs=['x'], src_indices=(1, 0, 2))
+        p.run_model()
 
+        assert_near_equal(p.get_val('exec.B').ravel(), p.get_val('ivc.M')[:1].ravel())
+        assert_near_equal(p.get_val('exec.A').ravel(), p.get_val('ivc.M')[:1].ravel())
 
 class SrcIndicesFeatureTestCase(unittest.TestCase):
     def test_multi_promotes(self):
@@ -409,15 +397,15 @@ class SrcIndicesFeatureTestCase(unittest.TestCase):
         p = om.Problem()
         G = p.model.add_subsystem('G', om.Group())
 
-        # At the top level, we assume that the source has a shape of (3,3), and after we
-        # slice it with [:,:-1], lower levels will see their source having a shape of (3,2)
+        # At the top level, we assume that the source, and our input 'x', has a shape of (3,3),
+        # and after we slice it with [:,:-1], lower levels will see their source having a shape of (3,2)
         p.model.promotes('G', inputs=['x'], src_indices=om.slicer[:,:-1], src_shape=(3,3))
 
         # This specifies that G.x assumes a source shape of (3,2)
         G.set_input_defaults('x', src_shape=(3,2))
 
         g1 = G.add_subsystem('g1', om.Group(), promotes_inputs=['x'])
-        g1.add_subsystem('C1', om.ExecComp('y = 3*x', shape=3))
+        g1.add_subsystem('C1', om.ExecComp('y = 3*x', x=np.random.random(3), y=np.random.random(3)))
 
         # C1.x has a shape of 3, so we apply a slice of [:, 1] to our source which has a shape
         # of (3,2) to give us our final shape of 3.
@@ -428,16 +416,18 @@ class SrcIndicesFeatureTestCase(unittest.TestCase):
 
         # C2.x has a shape of 2, so we apply flat source indices of [1,5] to our source which has
         # a shape of (3,2) to give us our final shape of 2.
-        g2.promotes('C2', inputs=['x'], src_indices=[1,5], src_shape=(3,2))
+        g2.promotes('C2', inputs=['x'], src_indices=[1,5], flat_src_indices=True, src_shape=(3,2))
 
         p.setup()
 
         inp = np.arange(9).reshape((3,3)) + 1.
 
-        p.set_val('x', inp[:, :-1])
+        # 'x' at the top level has a shape of (3,3) because no src_indices have been applied yet
+        p.set_val('x', inp)
         p.run_model()
 
-        assert_near_equal(p['x'], inp[:, :-1])
+        assert_near_equal(p['x'], inp)
+        assert_near_equal(p['G.x'], inp[:, :-1])
         assert_near_equal(p['G.g1.C1.y'], inp[:, :-1][:, 1]*3.)
         assert_near_equal(p['G.g2.C2.y'], inp[:, :-1].flatten()[[1,5]]*2.)
 
@@ -474,7 +464,7 @@ class SrcIndicesMPITestCase(unittest.TestCase):
 
         reduced_inp = inp[:, :-1]
 
-        p.set_val('x', reduced_inp)
+        p.set_val('x', inp)
         p.run_model()
 
         if commsize == 1 or p.comm.rank == 0:
@@ -483,5 +473,137 @@ class SrcIndicesMPITestCase(unittest.TestCase):
             assert_near_equal(p['par.g2.C2.y'], reduced_inp.flatten()[[1,5]]*2.)
 
 
+class SrcIndicesSerialMultipoint2(unittest.TestCase):
+
+    def test_multipoint2(self):
+        p = om.Problem()
+        par = p.model.add_subsystem('par', om.ParallelGroup())
+
+        g1 = par.add_subsystem('g1', om.Group(), promotes_inputs=['x'])
+        g1.add_subsystem('C1', om.ExecComp('y = 3*x', shape=1))
+        g1.promotes('C1', inputs=['x'], src_indices=[0], src_shape=(2,))
+
+        g2 = par.add_subsystem('g2', om.Group(), promotes_inputs=['x'])
+        g2.add_subsystem('C2', om.ExecComp('y = 2*x', shape=1))
+        g2.promotes('C2', inputs=['x'], src_indices=[1], src_shape=(2,))
+
+        p.model.set_input_defaults('par.x', val=[7., -5.])
+
+        p.setup()
+        p.run_model()
+
+
+class Multipoint2TooManyProcs(SrcIndicesSerialMultipoint2):
+    N_PROCS = 3
+
+
+class SrcIndicesSerialMultipoint3(unittest.TestCase):
+
+    def test_multipoint2(self):
+        p = om.Problem()
+        par = p.model.add_subsystem('par', om.ParallelGroup())
+
+        g1 = par.add_subsystem('g1', om.Group(), promotes_inputs=['x'])
+        g1.add_subsystem('C1', om.ExecComp('y = 3*x', shape=1))
+        g1.promotes('C1', inputs=['x'], src_indices=[0], src_shape=(3,))
+
+        g2 = par.add_subsystem('g2', om.Group(), promotes_inputs=['x'])
+        g2.add_subsystem('C2', om.ExecComp('y = 2*x', shape=1))
+        g2.promotes('C2', inputs=['x'], src_indices=[1], src_shape=(3,))
+
+        g3 = par.add_subsystem('g3', om.Group(), promotes_inputs=['x'])
+        g3.add_subsystem('C3', om.ExecComp('y = 5*x', shape=1))
+        g3.promotes('C3', inputs=['x'], src_indices=[2], src_shape=(3,))
+
+        p.model.set_input_defaults('par.x', val=[7., -5., 2.])
+
+        p.setup()
+        p.run_model()
+
+        assert_check_totals(p.check_totals(of=['par.g1.C1.y', 'par.g2.C2.y', 'par.g3.C3.y'], wrt=['par.x']))
+
+
+class Multipoint3SameProcs(SrcIndicesSerialMultipoint3):
+    N_PROCS = 3
+
+
+class Multipoint3TooManyProcs(SrcIndicesSerialMultipoint3):
+    N_PROCS = 4
+
+
+class DoubleNestedParallelMultipointTestCase(unittest.TestCase):
+    N_PROCS = 6
+
+    def test_nested_pars(self):
+        p = om.Problem()
+        par = p.model.add_subsystem('par', om.ParallelGroup(), promotes_inputs=['x'])
+
+        G1 = par.add_subsystem('G1', om.Group(), promotes_inputs=['x'])
+        G1p = G1.add_subsystem('G1p', om.ParallelGroup(), promotes_inputs=['x'])
+        G1p.add_subsystem('C1_1', om.ExecComp('y = 3.0*x'))
+        G1p.add_subsystem('C1_2', om.ExecComp('y = -3.0*x'))
+        G1p.promotes('C1_1', inputs=['x'], src_indices=[0])
+        G1p.promotes('C1_2', inputs=['x'], src_indices=[0])
+
+        G2 = par.add_subsystem('G2', om.Group(), promotes_inputs=['x'])
+        G2p = G2.add_subsystem('G2p', om.ParallelGroup(), promotes_inputs=['x'])
+        G2p.add_subsystem('C2_1', om.ExecComp('y = 5.0*x'))
+        G2p.add_subsystem('C2_2', om.ExecComp('y = -5.0*x'))
+        G2p.promotes('C2_1', inputs=['x'], src_indices=[1])
+        G2p.promotes('C2_2', inputs=['x'], src_indices=[1])
+
+        G3 = par.add_subsystem('G3', om.Group(), promotes_inputs=['x'], max_procs=1)  # no nested parallel group here
+        G3s = G3.add_subsystem('G3s', om.Group(), promotes_inputs=['x'])
+        G3s.add_subsystem('C3_1', om.ExecComp('y = 7.0*x'))
+        G3s.add_subsystem('C3_2', om.ExecComp('y = -7.0*x'))
+        G3s.promotes('C3_1', inputs=['x'], src_indices=[2])
+        G3s.promotes('C3_2', inputs=['x'], src_indices=[2])
+
+        par.set_input_defaults('x', val=[.5, 1.5, 2.5])
+
+        p.setup()
+        p.run_model()
+
+        assert_check_totals(p.check_totals(of=['par.G1.G1p.C1_1.y', 'par.G1.G1p.C1_2.y',
+                                               'par.G2.G2p.C2_1.y', 'par.G2.G2p.C2_2.y',
+                                               'par.G3.G3s.C3_1.y', 'par.G3.G3s.C3_2.y'], wrt=['x']))
+
+
+class TestNestedInputDefaults(unittest.TestCase):
+    N_PROCS = 3
+
+    def test_nested(self):
+        # this was failing as reported in github issue #2441
+        # It passes if the two setup() calls don't raise any exceptions.
+
+        class Grp(om.Group):
+            def setup(self):
+                self.add_subsystem("lin", om.ExecComp("y = x" ), promotes_inputs=["x"])
+                self.add_subsystem("quad", om.ExecComp("z = x*x"), promotes_inputs=["x"])
+                # self.set_input_defaults("x", 0.85)
+
+        class Vec(om.Group):
+            def initialize(self):
+                self.options.declare("num", default=1)
+            def setup(self):
+                n = self.options["num"]
+
+                self.set_input_defaults("x", np.full(n, 0.85))
+
+                for node in range(n):
+                    name = f"comp_{node}"
+                    self.add_subsystem(name, Grp())
+                    self.promotes(name, inputs=["x"], src_indices=[node])
+
+        # This one seems to require that set_input_defaults (on line 8) is NOT called in Comp.setup()
+        n = 3
+        p = om.Problem(Vec(num=n))
+        p.setup()
+
+        # This one seems to require that set_input_defaults (on line 8) IS called in Comp.setup()
+        p2 = om.Problem(Grp())
+        p2.setup()
+
+
 if __name__ == '__main__':
     unittest.main()
