@@ -26,7 +26,7 @@ import copy
 import numpy as np
 from pyDOE2 import lhs
 
-import openmdao
+from openmdao.core.constants import INF_BOUND
 from openmdao.core.driver import Driver, RecordingDebugging
 from openmdao.utils.concurrent import concurrent_eval
 from openmdao.utils.mpi import MPI
@@ -37,8 +37,15 @@ class SimpleGADriver(Driver):
     """
     Driver for a simple genetic algorithm.
 
+    Parameters
+    ----------
+    **kwargs : dict of keyword arguments
+        Keyword arguments that will be mapped into the Driver options.
+
     Attributes
     ----------
+    _problem_comm : MPI.Comm or None
+        The MPI communicator for the Problem.
     _concurrent_pop_size : int
         Number of points to run concurrently when model is a parallel one.
     _concurrent_color : int
@@ -55,11 +62,6 @@ class SimpleGADriver(Driver):
     def __init__(self, **kwargs):
         """
         Initialize the SimpleGADriver driver.
-
-        Parameters
-        ----------
-        **kwargs : dict of keyword arguments
-            Keyword arguments that will be mapped into the Driver options.
         """
         super().__init__(**kwargs)
 
@@ -176,6 +178,8 @@ class SimpleGADriver(Driver):
         MPI.Comm or <FakeComm> or None
             The communicator for the Problem model.
         """
+        self._problem_comm = comm
+
         procs_per_model = self.options['procs_per_model']
         if MPI and self.options['run_parallel']:
 
@@ -200,6 +204,31 @@ class SimpleGADriver(Driver):
         self._concurrent_color = 0
         return comm
 
+    def _setup_recording(self):
+        """
+        Set up case recording.
+        """
+        if MPI:
+            run_parallel = self.options['run_parallel']
+            procs_per_model = self.options['procs_per_model']
+
+            for recorder in self._rec_mgr:
+                if run_parallel:
+                    # write cases only on procs up to the number of parallel models
+                    # (i.e. on the root procs for the cases)
+                    if procs_per_model == 1:
+                        recorder.record_on_process = True
+                    else:
+                        size = self._problem_comm.size // procs_per_model
+                        if self._problem_comm.rank < size:
+                            recorder.record_on_process = True
+
+                elif self._problem_comm.rank == 0:
+                    # if not running cases in parallel, then just record on proc 0
+                    recorder.record_on_process = True
+
+        super()._setup_recording()
+
     def _get_name(self):
         """
         Get name of current Driver.
@@ -217,7 +246,7 @@ class SimpleGADriver(Driver):
 
         Returns
         -------
-        boolean
+        bool
             Failure flag; True if failed to converge, False is successful.
         """
         model = self._problem().model
@@ -247,7 +276,7 @@ class SimpleGADriver(Driver):
         for name, meta in desvars.items():
             if name in self._designvars_discrete:
                 val = desvar_vals[name]
-                if np.isscalar(val):
+                if np.ndim(val) == 0:
                     size = 1
                 else:
                     size = len(val)
@@ -259,7 +288,7 @@ class SimpleGADriver(Driver):
         lower_bound = np.empty((count, ))
         upper_bound = np.empty((count, ))
         outer_bound = np.full((count, ), np.inf)
-        bits = np.empty((count, ), dtype=np.int)
+        bits = np.empty((count, ), dtype=np.int_)
         x0 = np.empty(count)
 
         # Figure out bounds vectors and initial design vars
@@ -393,9 +422,9 @@ class SimpleGADriver(Driver):
         Returns
         -------
         float
-            Objective value
+            Objective value.
         bool
-            Success flag, True if successful
+            Success flag, True if successful.
         int
             Case number, used for identification when run in parallel.
         """
@@ -426,7 +455,7 @@ class SimpleGADriver(Driver):
             self.set_design_var(name, x[i:j])
 
         # a very large number, but smaller than the result of nan_to_num in Numpy
-        almost_inf = openmdao.INF_BOUND
+        almost_inf = INF_BOUND
 
         # Execute the model
         with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
@@ -502,6 +531,16 @@ class GeneticAlgorithm(object):
     This is the Simple Genetic Algorithm implementation based on 2009 AAE550: MDO Lecture notes of
     Prof. William A. Crossley. It can be used standalone or as part of the OpenMDAO Driver.
 
+    Parameters
+    ----------
+    objfun : function
+        Objective callback function.
+    comm : MPI communicator or None
+        The MPI communicator that will be used objective evaluation for each generation.
+    model_mpi : None or tuple
+        If the model in objfun is also parallel, then this will contain a tuple with the the
+        total number of population points to evaluate concurrently, and the color of the point
+        to evaluate on this rank.
 
     Attributes
     ----------
@@ -531,17 +570,6 @@ class GeneticAlgorithm(object):
     def __init__(self, objfun, comm=None, model_mpi=None):
         """
         Initialize genetic algorithm object.
-
-        Parameters
-        ----------
-        objfun : function
-            Objective callback function.
-        comm : MPI communicator or None
-            The MPI communicator that will be used objective evaluation for each generation.
-        model_mpi : None or tuple
-            If the model in objfun is also parallel, then this will contain a tuple with the the
-            total number of population points to evaluate concurrently, and the color of the point
-            to evaluate on this rank.
         """
         self.objfun = objfun
         self.comm = comm
@@ -561,7 +589,7 @@ class GeneticAlgorithm(object):
         Parameters
         ----------
         x0 : ndarray
-            Initial design values
+            Initial design values.
         vlb : ndarray
             Lower bounds array.
         vub : ndarray
@@ -578,14 +606,14 @@ class GeneticAlgorithm(object):
         random_state : np.random.RandomState, int
             Random state (or seed-number) which controls the seed and random draws.
         Pm : float or None
-            Mutation rate
+            Mutation rate.
         Pc : float
-            Crossover rate
+            Crossover rate.
 
         Returns
         -------
         ndarray
-            Best design point
+            Best design point.
         float
             Objective value at best design point.
         int
@@ -768,7 +796,7 @@ class GeneticAlgorithm(object):
         Parameters
         ----------
         old_gen : ndarray
-            Points in current generation
+            Points in current generation.
         fitness : ndarray
             Objective value of each point.
 
@@ -800,7 +828,7 @@ class GeneticAlgorithm(object):
         Parameters
         ----------
         old_gen : ndarray
-            Points in current generation
+            Points in current generation.
         obj_val : ndarray
             Objective value of each point.
 
@@ -840,8 +868,7 @@ class GeneticAlgorithm(object):
         Parameters
         ----------
         old_gen : ndarray
-            Points in current generation
-
+            Points in current generation.
         Pc : float
             Probability of crossover.
 
@@ -874,7 +901,7 @@ class GeneticAlgorithm(object):
         Parameters
         ----------
         current_gen : ndarray
-            Points in current generation
+            Points in current generation.
 
         Pm : float
             Probability of mutation.
@@ -973,7 +1000,7 @@ class GeneticAlgorithm(object):
         interval = (vub - vlb) / (2**bits - 1)
         x = np.maximum(x, vlb)
         x = np.minimum(x, vub)
-        x = np.round((x - vlb) / interval).astype(np.int)
+        x = np.round((x - vlb) / interval).astype(np.int_)
         byte_str = [("0" * b + bin(i)[2:])[-b:] for i, b in zip(x, bits)]
         result = np.array([int(c) for s in byte_str for c in s])
         if self.gray_code:
