@@ -6,25 +6,30 @@ formulating and solving nonlinear constrained optimization problems, with
 additional MPI capability.
 """
 
-from collections import OrderedDict
+import sys
 import json
 import signal
+from distutils.version import LooseVersion
 
 import numpy as np
 from scipy.sparse import coo_matrix
 
 try:
-    from pyoptsparse import Optimization
-except ImportError:
+    import pyoptsparse
+    Optimization = pyoptsparse.Optimization
+except Exception:
     Optimization = None
+    pyoptsparse = None
 
 from openmdao.core.constants import INT_DTYPE
 from openmdao.core.analysis_error import AnalysisError
 from openmdao.core.driver import Driver, RecordingDebugging
 import openmdao.utils.coloring as c_mod
-from openmdao.utils.class_util import weak_method_wrapper
+from openmdao.utils.class_util import WeakMethodWrapper
 from openmdao.utils.mpi import FakeComm
-from openmdao.warnings import issue_warning, DerivativesWarning
+from openmdao.utils.om_warnings import issue_warning, DerivativesWarning
+from openmdao.utils.general_utils import _src_or_alias_name
+from openmdao.utils.mpi import MPI
 
 
 # names of optimizers that use gradients
@@ -114,6 +119,11 @@ class pyOptSparseDriver(Driver):
 
         two_sided_constraints
 
+    Parameters
+    ----------
+    **kwargs : dict of keyword arguments
+        Keyword arguments that will be mapped into the Driver options.
+
     Attributes
     ----------
     fail : bool
@@ -129,8 +139,9 @@ class pyOptSparseDriver(Driver):
         Pyopt_sparse solution object.
     _check_jac : bool
         Used internally to control when to perform singular checks on computed total derivs.
-    _exc_info : None or <Exception>
-        Cached exception that was raised in the _objfunc or _gradfunc callbacks.
+    _exc_info : 3 item tuple
+        Storage for exception and traceback information for exception that was raised in the
+        _objfunc or _gradfunc callbacks.
     _in_user_function :bool
         This is set to True at the start of a pyoptsparse callback to _objfunc and _gradfunc, and
         restored to False at the finish of each callback.
@@ -148,11 +159,6 @@ class pyOptSparseDriver(Driver):
     def __init__(self, **kwargs):
         """
         Initialize pyopt.
-
-        Parameters
-        ----------
-        **kwargs : dict of keyword arguments
-            Keyword arguments that will be mapped into the Driver options.
         """
         if Optimization is None:
             raise RuntimeError('pyOptSparseDriver is not available, pyOptsparse is not installed.')
@@ -206,9 +212,17 @@ class pyOptSparseDriver(Driver):
                              desc='Name of optimizers to use')
         self.options.declare('title', default='Optimization using pyOpt_sparse',
                              desc='Title of this optimization run')
+        self.options.declare('print_opt_prob', types=bool, default=False,
+                             desc='Print the opt problem summary before running the optimization')
         self.options.declare('print_results', types=bool, default=True,
                              desc='Print pyOpt results if True')
         self.options.declare('gradient method', default='openmdao',
+                             values={'openmdao', 'pyopt_fd', 'snopt_fd'},
+                             desc='Finite difference implementation to use',
+                             deprecation=(f"'gradient method' is not a valid python name and will "
+                                          "raise an exception in a future release.  Use "
+                                          "'gradient_method' instead.", 'gradient_method'))
+        self.options.declare('gradient_method', default='openmdao',
                              values={'openmdao', 'pyopt_fd', 'snopt_fd'},
                              desc='Finite difference implementation to use')
         self.options.declare('user_terminate_signal', default=DEFAULT_SIGNAL, allow_none=True,
@@ -228,8 +242,9 @@ class pyOptSparseDriver(Driver):
         self.options.declare('user_teriminate_signal', default=None, allow_none=True,
                              desc='OS signal that triggers a clean user-termination. Only SNOPT'
                              'supports this option.',
-                             deprecation="The option 'user_teriminate_signal' was misspelled and "
-                             "will be deprecated. Please use 'user_terminate_signal' instead.")
+                             deprecation=("The option 'user_teriminate_signal' was misspelled and "
+                                          "will be deprecated. Please use 'user_terminate_signal' "
+                                          "instead.", 'user_terminate_signal'))
 
     def _setup_driver(self, problem):
         """
@@ -256,9 +271,9 @@ class pyOptSparseDriver(Driver):
         self._setup_tot_jac_sparsity()
 
         # Handle deprecated option.
-        if self.options._dict['user_teriminate_signal']['value'] is not None:
+        if self.options._dict['user_teriminate_signal']['val'] is not None:
             self.options['user_terminate_signal'] = \
-                self.options._dict['user_teriminate_signal']['value']
+                self.options._dict['user_teriminate_signal']['val']
 
     def run(self):
         """
@@ -269,7 +284,7 @@ class pyOptSparseDriver(Driver):
 
         Returns
         -------
-        boolean
+        bool
             Failure flag; True if failed to converge, False is successful.
         """
         problem = self._problem()
@@ -316,21 +331,25 @@ class pyOptSparseDriver(Driver):
                     issue_warning(msg, prefix=self.msginfo, category=DerivativesWarning)
 
         comm = None if isinstance(problem.comm, FakeComm) else problem.comm
-        opt_prob = Optimization(self.options['title'], weak_method_wrapper(self, '_objfunc'),
+        opt_prob = Optimization(self.options['title'], WeakMethodWrapper(self, '_objfunc'),
                                 comm=comm)
 
         # Add all design variables
-        input_meta = self._designvars
-        self._indep_list = indep_list = list(input_meta)
+        dv_meta = self._designvars
+        self._indep_list = indep_list = list(dv_meta)
         input_vals = self.get_design_var_values()
 
-        for name, meta in input_meta.items():
+        for name, meta in dv_meta.items():
             size = meta['global_size'] if meta['distributed'] else meta['size']
             opt_prob.addVarGroup(name, size, type='c',
                                  value=input_vals[name],
                                  lower=meta['lower'], upper=meta['upper'])
 
-        opt_prob.finalizeDesignVariables()
+        if not hasattr(pyoptsparse, '__version__') or \
+           LooseVersion(pyoptsparse.__version__) < LooseVersion('2.5.1'):
+            opt_prob.finalizeDesignVariables()
+        else:
+            opt_prob.finalize()
 
         # Add all objectives
         objs = self.get_objective_values()
@@ -364,22 +383,24 @@ class pyOptSparseDriver(Driver):
                 continue
             size = meta['global_size'] if meta['distributed'] else meta['size']
             lower = upper = meta['equals']
+            path = meta['source'] if meta['alias'] is not None else name
             if fwd:
-                wrt = [v for v in indep_list if name in relevant[input_meta[v]['ivc_source']]]
+                wrt = [v for v in indep_list if path in relevant[dv_meta[v]['source']]]
             else:
-                rels = relevant[name]
-                wrt = [v for v in indep_list if input_meta[v]['ivc_source'] in rels]
+                rels = relevant[path]
+                wrt = [v for v in indep_list if dv_meta[v]['source'] in rels]
 
             if meta['linear']:
                 jac = {w: _lin_jacs[name][w] for w in wrt}
                 opt_prob.addConGroup(name, size, lower=lower, upper=upper,
                                      linear=True, wrt=wrt, jac=jac)
             else:
-                if name in self._res_jacs:
-                    resjac = self._res_jacs[name]
-                    jac = {n: resjac[input_meta[n]['ivc_source']] for n in wrt}
+                if name in self._res_subjacs:
+                    resjac = self._res_subjacs[name]
+                    jac = {n: resjac[dv_meta[n]['source']] for n in wrt}
                 else:
                     jac = None
+
                 opt_prob.addConGroup(name, size, lower=lower, upper=upper, wrt=wrt, jac=jac)
                 self._quantities.append(name)
 
@@ -393,20 +414,22 @@ class pyOptSparseDriver(Driver):
             lower = meta['lower']
             upper = meta['upper']
 
+            path = meta['source'] if meta['alias'] is not None else name
+
             if fwd:
-                wrt = [v for v in indep_list if name in relevant[input_meta[v]['ivc_source']]]
+                wrt = [v for v in indep_list if path in relevant[dv_meta[v]['source']]]
             else:
-                rels = relevant[name]
-                wrt = [v for v in indep_list if input_meta[v]['ivc_source'] in rels]
+                rels = relevant[path]
+                wrt = [v for v in indep_list if dv_meta[v]['source'] in rels]
 
             if meta['linear']:
                 jac = {w: _lin_jacs[name][w] for w in wrt}
                 opt_prob.addConGroup(name, size, upper=upper, lower=lower,
                                      linear=True, wrt=wrt, jac=jac)
             else:
-                if name in self._res_jacs:
-                    resjac = self._res_jacs[name]
-                    jac = {n: resjac[input_meta[n]['ivc_source']] for n in wrt}
+                if name in self._res_subjacs:
+                    resjac = self._res_subjacs[name]
+                    jac = {n: resjac[dv_meta[n]['source']] for n in wrt}
                 else:
                     jac = None
                 opt_prob.addConGroup(name, size, upper=upper, lower=lower, wrt=wrt, jac=jac)
@@ -433,11 +456,17 @@ class pyOptSparseDriver(Driver):
         for option, value in self.opt_settings.items():
             opt.setOption(option, value)
 
+        # Print the pyoptsparse optimization problem summary before running the optimization.
+        # This allows users to confirm their optimization setup.
+        if self.options['print_opt_prob']:
+            if not MPI or model.comm.rank == 0:
+                print(opt_prob)
+
         self._exc_info = None
         try:
 
             # Execute the optimization problem
-            if self.options['gradient method'] == 'pyopt_fd':
+            if self.options['gradient_method'] == 'pyopt_fd':
 
                 # Use pyOpt's internal finite difference
                 # TODO: Need to get this from OpenMDAO
@@ -446,7 +475,7 @@ class pyOptSparseDriver(Driver):
                 sol = opt(opt_prob, sens='FD', sensStep=fd_step, storeHistory=self.hist_file,
                           hotStart=self.hotstart_file)
 
-            elif self.options['gradient method'] == 'snopt_fd':
+            elif self.options['gradient_method'] == 'snopt_fd':
                 if self.options['optimizer'] == 'SNOPT':
 
                     # Use SNOPT's internal finite difference
@@ -458,23 +487,26 @@ class pyOptSparseDriver(Driver):
 
                 else:
                     msg = "SNOPT's internal finite difference can only be used with SNOPT"
-                    self._exc_info = Exception(msg)
+                    self._exc_info = (Exception, Exception(msg), None)
             else:
 
                 # Use OpenMDAO's differentiator for the gradient
-                sol = opt(opt_prob, sens=weak_method_wrapper(self, '_gradfunc'),
+                sol = opt(opt_prob, sens=WeakMethodWrapper(self, '_gradfunc'),
                           storeHistory=self.hist_file, hotStart=self.hotstart_file)
 
-        except Exception as _:
+        except Exception as c:
             if not self._exc_info:
-                raise()
+                raise
 
         if self._exc_info:
-            raise self._exc_info
+            if self._exc_info[2] is None:
+                raise self._exc_info[1]
+            raise self._exc_info[1].with_traceback(self._exc_info[2])
 
         # Print results
         if self.options['print_results']:
-            print(sol)
+            if not MPI or model.comm.rank == 0:
+                print(sol)
 
         # Pull optimal parameters back into framework and re-run, so that
         # framework is left in the right final state
@@ -587,8 +619,8 @@ class pyOptSparseDriver(Driver):
                 rec.abs = 0.0
                 rec.rel = 0.0
 
-        except Exception as raised:
-            self._exc_info = raised
+        except Exception:
+            self._exc_info = sys.exc_info()
             fail = 1
             func_dict = {}
 
@@ -609,9 +641,10 @@ class pyOptSparseDriver(Driver):
         Parameters
         ----------
         dv_dict : dict
-            Dictionary of design variable values.
+            Dictionary of design variable values. All keys are sources.
         func_dict : dict
-            Dictionary of all functional variables evaluated at design point.
+            Dictionary of all functional variables evaluated at design point. Keys are
+            sources and aliases.
 
         Returns
         -------
@@ -658,16 +691,16 @@ class pyOptSparseDriver(Driver):
                 # if we don't convert to 'coo' here, pyoptsparse will do a
                 # conversion of our dense array into a fully dense 'coo', which is bad.
                 # TODO: look into getting rid of all of these conversions!
-                new_sens = OrderedDict()
-                res_jacs = self._res_jacs
+                new_sens = {}
+                res_subjacs = self._res_subjacs
                 for okey in func_dict:
-                    new_sens[okey] = newdv = OrderedDict()
-                    okey_src = self._responses[okey]['ivc_source']
+                    new_sens[okey] = newdv = {}
+                    osrc_or_alias = _src_or_alias_name(self._responses[okey])
                     for ikey in dv_dict:
-                        ikey_src = self._designvars[ikey]['ivc_source']
-                        if okey_src in res_jacs and ikey_src in res_jacs[okey_src]:
+                        ikey_src = self._designvars[ikey]['source']
+                        if osrc_or_alias in res_subjacs and ikey_src in res_subjacs[osrc_or_alias]:
                             arr = sens_dict[okey][ikey]
-                            coo = res_jacs[okey_src][ikey_src]
+                            coo = res_subjacs[osrc_or_alias][ikey_src]
                             row, col, data = coo['coo']
                             coo['coo'][2] = arr[row, col].flatten()
                             newdv[ikey] = coo
@@ -679,16 +712,16 @@ class pyOptSparseDriver(Driver):
                 # We need to cobble together a sens_dict of the correct size.
                 # Best we can do is return zeros.
 
-                sens_dict = OrderedDict()
+                sens_dict = {}
                 for okey, oval in func_dict.items():
-                    sens_dict[okey] = OrderedDict()
+                    sens_dict[okey] = {}
                     osize = len(oval)
                     for ikey, ival in dv_dict.items():
                         isize = len(ival)
                         sens_dict[okey][ikey] = np.zeros((osize, isize))
 
-        except Exception as raised:
-            self._exc_info = raised
+        except Exception:
+            self._exc_info = sys.exc_info()
             fail = 1
             sens_dict = {}
 
@@ -744,7 +777,7 @@ class pyOptSparseDriver(Driver):
             Current coloring.
         """
         total_sparsity = None
-        self._res_jacs = {}
+        self._res_subjacs = {}
         coloring = coloring if coloring is not None else self._get_static_coloring()
         if coloring is not None:
             total_sparsity = coloring.get_subjac_sparsity()
@@ -760,15 +793,18 @@ class pyOptSparseDriver(Driver):
         if total_sparsity is None:
             return
 
+        model = self._problem().model
         for res, resdict in total_sparsity.items():
+            if res in self._responses and self._responses[res]['alias'] is not None:
+                res = self._responses[res]['source']
             if res in self._objs:  # skip objectives
                 continue
-            self._res_jacs[res] = {}
+            self._res_subjacs[res] = {}
             for dv, (rows, cols, shape) in resdict.items():
                 rows = np.array(rows, dtype=INT_DTYPE)
                 cols = np.array(cols, dtype=INT_DTYPE)
 
-                self._res_jacs[res][dv] = {
+                self._res_subjacs[res][dv] = {
                     'coo': [rows, cols, np.zeros(rows.size)],
                     'shape': shape,
                 }
