@@ -2,21 +2,19 @@
 OpenMDAO Wrapper for the scipy.optimize.minimize family of local optimizers.
 """
 
-
 import sys
-from collections import OrderedDict
 from distutils.version import LooseVersion
 
 import numpy as np
 from scipy import __version__ as scipy_version
 from scipy.optimize import minimize
 
-import openmdao
+from openmdao.core.constants import INF_BOUND
 import openmdao.utils.coloring as coloring_mod
 from openmdao.core.driver import Driver, RecordingDebugging
-from openmdao.utils.general_utils import simple_warning
-from openmdao.utils.class_util import weak_method_wrapper
+from openmdao.utils.class_util import WeakMethodWrapper
 from openmdao.utils.mpi import MPI
+from openmdao.utils.om_warnings import issue_warning, DerivativesWarning
 
 # Optimizers in scipy.minimize
 _optimizers = {'Nelder-Mead', 'Powell', 'CG', 'BFGS', 'Newton-CG', 'L-BFGS-B',
@@ -82,6 +80,11 @@ class ScipyOptimizeDriver(Driver):
         equality_constraints
         inequality_constraints
 
+    Parameters
+    ----------
+    **kwargs : dict of keyword arguments
+        Keyword arguments that will be mapped into the Driver options.
+
     Attributes
     ----------
     fail : bool
@@ -92,11 +95,13 @@ class ScipyOptimizeDriver(Driver):
         Result returned from scipy.optimize call.
     opt_settings : dict
         Dictionary of solver-specific options. See the scipy.optimize.minimize documentation.
+    _check_jac : bool
+        Used internally to control when to perform singular checks on computed total derivs.
     _con_cache : dict
         Cached result of constraint evaluations because scipy asks for them in a separate function.
     _con_idx : dict
         Used for constraint bookkeeping in the presence of 2-sided constraints.
-    _grad_cache : OrderedDict
+    _grad_cache : {}
         Cached result of nonlinear constraint derivatives because scipy asks for them in a separate
         function.
     _exc_info : 3 item tuple
@@ -113,13 +118,8 @@ class ScipyOptimizeDriver(Driver):
     def __init__(self, **kwargs):
         """
         Initialize the ScipyOptimizeDriver.
-
-        Parameters
-        ----------
-        **kwargs : dict of keyword arguments
-            Keyword arguments that will be mapped into the Driver options.
         """
-        super(ScipyOptimizeDriver, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
         # What we support
         self.supports['inequality_constraints'] = True
@@ -132,9 +132,11 @@ class ScipyOptimizeDriver(Driver):
         self.supports['multiple_objectives'] = False
         self.supports['active_set'] = False
         self.supports['integer_design_vars'] = False
+        self.supports['distributed_design_vars'] = False
+        self.supports._read_only = True
 
         # The user places optimizer-specific settings in here.
-        self.opt_settings = OrderedDict()
+        self.opt_settings = {}
 
         self.result = None
         self._grad_cache = None
@@ -145,6 +147,7 @@ class ScipyOptimizeDriver(Driver):
         self._lincongrad_cache = None
         self.fail = False
         self.iter_count = 0
+        self._check_jac = False
         self._exc_info = None
 
         self.cite = CITATIONS
@@ -162,6 +165,15 @@ class ScipyOptimizeDriver(Driver):
                              desc='Maximum number of iterations.')
         self.options.declare('disp', True, types=bool,
                              desc='Set to False to prevent printing of Scipy convergence messages')
+        self.options.declare('singular_jac_behavior', default='warn',
+                             values=['error', 'warn', 'ignore'],
+                             desc='Defines behavior of a zero row/col check after first call to'
+                             'compute_totals:'
+                             'error - raise an error.'
+                             'warn - raise a warning.'
+                             "ignore - don't perform check.")
+        self.options.declare('singular_jac_tol', default=1e-16,
+                             desc='Tolerance for zero row/column check.')
 
     def _get_name(self):
         """
@@ -185,38 +197,33 @@ class ScipyOptimizeDriver(Driver):
         problem : <Problem>
             Pointer
         """
-        super(ScipyOptimizeDriver, self)._setup_driver(problem)
+        super()._setup_driver(problem)
         opt = self.options['optimizer']
 
+        self.supports._read_only = False
         self.supports['gradients'] = opt in _gradient_optimizers
         self.supports['inequality_constraints'] = opt in _constraint_optimizers
         self.supports['two_sided_constraints'] = opt in _constraint_optimizers
         self.supports['equality_constraints'] = opt in _eq_constraint_optimizers
+        self.supports._read_only = True
+        self._check_jac = self.options['singular_jac_behavior'] in ['error', 'warn']
 
         # Raises error if multiple objectives are not supported, but more objectives were defined.
         if not self.supports['multiple_objectives'] and len(self._objs) > 1:
             msg = '{} currently does not support multiple objectives.'
             raise RuntimeError(msg.format(self.msginfo))
 
-        # Since COBYLA does not support bounds, we
-        #   need to add to the _cons metadata for any bounds that
-        #   need to be translated into a constraint
+        # Since COBYLA does not support bounds, we need to add to the _cons metadata
+        # for any bounds that need to be translated into a constraint
         if opt == 'COBYLA':
             for name, meta in self._designvars.items():
                 lower = meta['lower']
                 upper = meta['upper']
-                if isinstance(lower, np.ndarray) or lower >= -openmdao.INF_BOUND \
-                        or isinstance(upper, np.ndarray) or upper <= openmdao.INF_BOUND:
-                    d = OrderedDict()
-                    d['lower'] = lower
-                    d['upper'] = upper
-                    d['equals'] = None
-                    d['indices'] = None
-                    d['adder'] = None
-                    d['scaler'] = None
-                    d['size'] = meta['size']
-                    d['linear'] = True
-                    self._cons[name] = d
+                if isinstance(lower, np.ndarray) or lower >= -INF_BOUND \
+                        or isinstance(upper, np.ndarray) or upper <= INF_BOUND:
+                    self._cons[name] = meta.copy()
+                    self._cons[name]['equals'] = None
+                    self._cons[name]['linear'] = True
 
     def run(self):
         """
@@ -224,7 +231,7 @@ class ScipyOptimizeDriver(Driver):
 
         Returns
         -------
-        boolean
+        bool
             Failure flag; True if failed to converge, False is successful.
         """
         problem = self._problem()
@@ -244,16 +251,17 @@ class ScipyOptimizeDriver(Driver):
         desvar_vals = self.get_design_var_values()
         self._dvlist = list(self._designvars)
 
-        # maxiter and disp get passsed into scipy with all the other options.
+        # maxiter and disp get passed into scipy with all the other options.
         if 'maxiter' not in self.opt_settings:  # lets you override the value in options
             self.opt_settings['maxiter'] = self.options['maxiter']
         self.opt_settings['disp'] = self.options['disp']
 
         # Size Problem
-        nparam = 0
-        for param in self._designvars.values():
-            nparam += param['size']
-        x_init = np.empty(nparam)
+        ndesvar = 0
+        for desvar in self._designvars.values():
+            size = desvar['global_size'] if desvar['distributed'] else desvar['size']
+            ndesvar += size
+        x_init = np.empty(ndesvar)
 
         # Initial Design Vars
         i = 0
@@ -264,7 +272,7 @@ class ScipyOptimizeDriver(Driver):
             bounds = None
 
         for name, meta in self._designvars.items():
-            size = meta['size']
+            size = meta['global_size'] if meta['distributed'] else meta['size']
             x_init[i:i + size] = desvar_vals[name]
             i += size
 
@@ -311,7 +319,10 @@ class ScipyOptimizeDriver(Driver):
 
         if opt in _constraint_optimizers:
             for name, meta in self._cons.items():
-                size = meta['size']
+                if meta['indices'] is not None:
+                    meta['size'] = size = meta['indices'].indexed_src_size
+                else:
+                    size = meta['global_size'] if meta['distributed'] else meta['size']
                 upper = meta['upper']
                 lower = meta['lower']
                 equals = meta['equals']
@@ -348,10 +359,10 @@ class ScipyOptimizeDriver(Driver):
                         # TODO linear constraint if meta['linear']
                         # TODO add option for Hessian
                         con = NonlinearConstraint(
-                            fun=signature_extender(weak_method_wrapper(self, '_con_val_func'),
+                            fun=signature_extender(WeakMethodWrapper(self, '_con_val_func'),
                                                    args),
                             lb=lb, ub=ub,
-                            jac=signature_extender(weak_method_wrapper(self, '_congradfunc'), args))
+                            jac=signature_extender(WeakMethodWrapper(self, '_congradfunc'), args))
                         constraints.append(con)
                 else:  # Type of constraints is list of dict
                     # Loop over every index separately,
@@ -362,9 +373,9 @@ class ScipyOptimizeDriver(Driver):
                             con_dict['type'] = 'eq'
                         else:
                             con_dict['type'] = 'ineq'
-                        con_dict['fun'] = weak_method_wrapper(self, '_confunc')
+                        con_dict['fun'] = WeakMethodWrapper(self, '_confunc')
                         if opt in _constraint_grad_optimizers:
-                            con_dict['jac'] = weak_method_wrapper(self, '_congradfunc')
+                            con_dict['jac'] = WeakMethodWrapper(self, '_congradfunc')
                         con_dict['args'] = [name, False, j]
                         constraints.append(con_dict)
 
@@ -374,15 +385,15 @@ class ScipyOptimizeDriver(Driver):
                         if isinstance(lower, np.ndarray):
                             lower = lower[j]
 
-                        dblcon = (upper < openmdao.INF_BOUND) and (lower > -openmdao.INF_BOUND)
+                        dblcon = (upper < INF_BOUND) and (lower > -INF_BOUND)
 
                         # Add extra constraint if double-sided
                         if dblcon:
                             dcon_dict = {}
                             dcon_dict['type'] = 'ineq'
-                            dcon_dict['fun'] = weak_method_wrapper(self, '_confunc')
+                            dcon_dict['fun'] = WeakMethodWrapper(self, '_confunc')
                             if opt in _constraint_grad_optimizers:
-                                dcon_dict['jac'] = weak_method_wrapper(self, '_congradfunc')
+                                dcon_dict['jac'] = WeakMethodWrapper(self, '_congradfunc')
                             dcon_dict['args'] = [name, True, j]
                             constraints.append(dcon_dict)
 
@@ -421,14 +432,17 @@ class ScipyOptimizeDriver(Driver):
                 if info['coloring'] is not None:
                     pct = info['coloring']._solves_info()[-1]
                     if info['min_improve_pct'] > pct:
-                        info['coloring'] = info['static'] = info['dynamic'] = None
-                        simple_warning("%s: Coloring was deactivated.  Improvement of %.1f%% was "
-                                       "less than min allowed (%.1f%%)." %
-                                       (self.msginfo, pct, info['min_improve_pct']))
+                        info['coloring'] = info['static'] = None
+                        msg = f"Coloring was deactivated.  Improvement of {pct:.1f}% was less " \
+                              f"than min allowed ({info['min_improve_pct']:.1f}%)."
+                        issue_warning(msg, prefix=self.msginfo, category=DerivativesWarning)
 
         # optimize
         try:
             if opt in _optimizers:
+                if self._problem().comm.rank != 0:
+                    self.opt_settings['disp'] = False
+
                 result = minimize(self._objfunc, x_init,
                                   # args=(),
                                   method=opt,
@@ -489,9 +503,9 @@ class ScipyOptimizeDriver(Driver):
             elif opt == 'shgo':
                 from scipy.optimize import shgo
                 kwargs = dict()
-                for param in ('minimizer_kwargs', 'sampling_method ', 'n', 'iters'):
-                    if param in self.opt_settings:
-                        kwargs[param] = self.opt_settings[param]
+                for option in ('minimizer_kwargs', 'sampling_method ', 'n', 'iters'):
+                    if option in self.opt_settings:
+                        kwargs[option] = self.opt_settings[option]
                 # Set the Jacobian and the Hessian to the value calculated in OpenMDAO
                 if 'minimizer_kwargs' not in kwargs or kwargs['minimizer_kwargs'] is None:
                     kwargs['minimizer_kwargs'] = {}
@@ -511,9 +525,7 @@ class ScipyOptimizeDriver(Driver):
         # If an exception was swallowed in one of our callbacks, we want to raise it
         # rather than the cryptic message from scipy.
         except Exception as msg:
-            if self._exc_info is not None:
-                self._reraise()
-            else:
+            if self._exc_info is None:
                 raise
 
         if self._exc_info is not None:
@@ -524,18 +536,21 @@ class ScipyOptimizeDriver(Driver):
         if hasattr(result, 'success'):
             self.fail = False if result.success else True
             if self.fail:
-                print('Optimization FAILED.')
-                print(result.message)
-                print('-' * 35)
+                if self._problem().comm.rank == 0:
+                    print('Optimization FAILED.')
+                    print(result.message)
+                    print('-' * 35)
 
             elif self.options['disp']:
-                print('Optimization Complete')
-                print('-' * 35)
+                if self._problem().comm.rank == 0:
+                    print('Optimization Complete')
+                    print('-' * 35)
         else:
             self.fail = True  # It is not known, so the worst option is assumed
-            print('Optimization Complete (success not known)')
-            print(result.message)
-            print('-' * 35)
+            if self._problem().comm.rank == 0:
+                print('Optimization Complete (success not known)')
+                print(result.message)
+                print('-' * 35)
 
         return self.fail
 
@@ -548,7 +563,7 @@ class ScipyOptimizeDriver(Driver):
         Parameters
         ----------
         x_new : ndarray
-            Array containing parameter values at new design point.
+            Array containing input values at new design point.
 
         Returns
         -------
@@ -559,7 +574,7 @@ class ScipyOptimizeDriver(Driver):
 
         try:
 
-            # Pass in new parameters
+            # Pass in new inputs
             i = 0
             if MPI:
                 model.comm.Bcast(x_new, root=0)
@@ -580,7 +595,7 @@ class ScipyOptimizeDriver(Driver):
             self._con_cache = self.get_constraint_values()
 
         except Exception as msg:
-            self._exc_info = msg
+            self._exc_info = sys.exc_info()
             return 0
 
         # print("Functions calculated")
@@ -599,8 +614,8 @@ class ScipyOptimizeDriver(Driver):
         Parameters
         ----------
         x_new : ndarray
-            Array containing parameter values at new design point.
-        name : string
+            Array containing input values at new design point.
+        name : str
             Name of the constraint to be evaluated.
         dbl : bool
             True if double sided constraint.
@@ -624,8 +639,8 @@ class ScipyOptimizeDriver(Driver):
         Parameters
         ----------
         x_new : ndarray
-            Array containing parameter values at new design point.
-        name : string
+            Array containing input values at new design point.
+        name : str
             Name of the constraint to be evaluated.
         dbl : bool
             True if double sided constraint.
@@ -660,7 +675,7 @@ class ScipyOptimizeDriver(Driver):
         if isinstance(lower, np.ndarray):
             lower = lower[idx]
 
-        if dbl or (lower <= -openmdao.INF_BOUND):
+        if dbl or (lower <= -INF_BOUND):
             return upper - cons[name][idx]
         else:
             return cons[name][idx] - lower
@@ -674,20 +689,27 @@ class ScipyOptimizeDriver(Driver):
         Parameters
         ----------
         x_new : ndarray
-            Array containing parameter values at new design point.
+            Array containing input values at new design point.
 
         Returns
         -------
         ndarray
-            Gradient of objective with respect to parameter array.
+            Gradient of objective with respect to input array.
         """
         try:
             grad = self._compute_totals(of=self._obj_and_nlcons, wrt=self._dvlist,
                                         return_format='array')
             self._grad_cache = grad
 
+            # First time through, check for zero row/col.
+            if self._check_jac:
+                raise_error = self.options['singular_jac_behavior'] == 'error'
+                self._total_jac.check_total_jac(raise_error=raise_error,
+                                                tol=self.options['singular_jac_tol'])
+                self._check_jac = False
+
         except Exception as msg:
-            self._exc_info = msg
+            self._exc_info = sys.exc_info()
             return np.array([[]])
 
         # print("Gradients calculated for objective")
@@ -706,8 +728,8 @@ class ScipyOptimizeDriver(Driver):
         Parameters
         ----------
         x_new : ndarray
-            Array containing parameter values at new design point.
-        name : string
+            Array containing input values at new design point.
+        name : str
             Name of the constraint to be evaluated.
         dbl : bool
             Denotes if a constraint is double-sided or not.
@@ -717,7 +739,7 @@ class ScipyOptimizeDriver(Driver):
         Returns
         -------
         float
-            Gradient of the constraint function wrt all params.
+            Gradient of the constraint function wrt all inputs.
         """
         if self._exc_info is not None:
             self._reraise()
@@ -744,7 +766,7 @@ class ScipyOptimizeDriver(Driver):
         if isinstance(lower, np.ndarray):
             lower = lower[idx]
 
-        if dbl or (lower <= -openmdao.INF_BOUND):
+        if dbl or (lower <= -INF_BOUND):
             return -grad[grad_idx, :]
         else:
             return grad[grad_idx, :]
@@ -753,8 +775,7 @@ class ScipyOptimizeDriver(Driver):
         """
         Reraise any exception encountered when scipy calls back into our method.
         """
-        exc = self._exc_info
-        raise exc
+        raise self._exc_info[1].with_traceback(self._exc_info[2])
 
 
 def signature_extender(fcn, extra_args):
@@ -773,7 +794,7 @@ def signature_extender(fcn, extra_args):
     fcn : callable
         Function, which takes the design vector as the first argument.
     extra_args : tuple or list
-        Extra arguments for the function
+        Extra arguments for the function.
 
     Returns
     -------
