@@ -4,6 +4,7 @@ from collections import defaultdict
 from distutils.version import LooseVersion
 
 import numpy as np
+import pickle
 
 from openmdao.core.group import Group
 from openmdao.core.component import Component
@@ -14,9 +15,10 @@ from openmdao.utils.logger_utils import get_logger
 from openmdao.utils.class_util import overrides_method
 from openmdao.utils.mpi import MPI
 from openmdao.utils.hooks import _register_hook
-from openmdao.utils.general_utils import printoptions, simple_warning, ignore_errors
-from openmdao.utils.units import convert_units, _has_val_mismatch
+from openmdao.utils.general_utils import printoptions, ignore_errors
+from openmdao.utils.units import _has_val_mismatch
 from openmdao.utils.file_utils import _load_and_exec
+from openmdao.utils.om_warnings import issue_warning, SetupWarning
 
 
 _UNSET = object()
@@ -176,9 +178,9 @@ def _get_used_before_calc_subs(group, input_srcs):
                     (parallel_solver[src_sys] not in ["NL: NLBJ", "NL: Newton", "BROYDEN"]) and
                     src_sys == tgt_sys and
                     not hierarchy_check):
-                simple_warning("Need to attach NonlinearBlockJac, NewtonSolver, or BroydenSolver "
-                               "to '%s' when connecting components inside parallel "
-                               "groups" % (src_sys))
+                msg = f"Need to attach NonlinearBlockJac, NewtonSolver, or BroydenSolver " \
+                      f"to '{src_sys}' when connecting components inside parallel groups"
+                issue_warning(msg, category=SetupWarning)
                 ubcs[tgt_abs.rsplit('.', 1)[0]].add(src_abs.rsplit('.', 1)[0])
             if (src_sys in allsubs and tgt_sys in allsubs and
                     (allsubs[src_sys].index > allsubs[tgt_sys].index)):
@@ -367,6 +369,8 @@ def _check_auto_ivc_warnings(problem, logger):
     ----------
     problem : <Problem>
         The problem being checked.
+    logger : object
+        The object that manages logging output.
     """
     if hasattr(problem.model, "_auto_ivc_warnings"):
         for i in problem.model._auto_ivc_warnings:
@@ -503,6 +507,66 @@ def _check_missing_recorders(problem, logger):
     logger.warning(msg)
 
 
+def _check_unserializable_options(problem, logger, check_recordable=True):
+    """
+    Check if there are any options that are not serializable, and therefore won't be recorded.
+
+    Parameters
+    ----------
+    problem : <Problem>
+        The problem being checked.
+    logger : object
+        The object that manages logging output.
+    check_recordable : bool
+        If False, warn about all unserializable options.
+        If True, warn only about unserializable options that do not have 'recordable' set to False.
+    """
+    from openmdao.recorders.case_recorder import PICKLE_VER
+
+    def _check_opts(obj, name=None):
+        if obj:
+            for key, val in obj.options.items():
+                try:
+                    pickle.dumps(val, PICKLE_VER)
+                except Exception:
+                    name_str = name + " " if name else ""
+                    if obj.options._dict[key]['recordable']:
+                        msg = f"{obj.msginfo}: {name_str}option '{key}' is not serializable " \
+                              "(cannot be pickled) but 'recordable=False' has not been set. " \
+                              f"No options will be recorded for this {obj.__class__.__name__} " \
+                              "unless 'recordable' is set to False for this option."
+                        logger.warning(msg)
+                    elif not check_recordable:
+                        msg = f"{obj.msginfo}: {name_str}option '{key}' is not serializable " \
+                              "(cannot be pickled) and will not be recorded."
+                        logger.warning(msg)
+
+    # check options for all for Systems and Solvers
+    for system in problem.model.system_iter(include_self=True, recurse=True):
+        _check_opts(system)
+        _check_opts(system.linear_solver, 'linear_solver')
+
+        nl = system.nonlinear_solver
+        if nl:
+            _check_opts(nl, 'nonlinear_solver')
+            for name in ('linear_solver', 'linesearch'):
+                _check_opts(getattr(nl, name, None), name)
+
+
+def _check_all_unserializable_options(problem, logger):
+    """
+    Check if there are any options that are not serializable, and therefore won't be recorded.
+
+    Parameters
+    ----------
+    problem : <Problem>
+        The problem being checked.
+    logger : object
+        The object that manages logging output.
+    """
+    _check_unserializable_options(problem, logger, False)
+
+
 def _get_promoted_connected_ins(g):
     """
     Find all inputs that are promoted above the level where they are explicitly connected.
@@ -577,6 +641,7 @@ _default_checks = {
     'solvers': _check_solvers,
     'dup_inputs': _check_dup_comp_inputs,
     'missing_recorders': _check_missing_recorders,
+    'unserializable_options': _check_unserializable_options,
     'comp_has_no_outputs': _check_comp_has_no_outputs,
     'auto_ivc_warnings': _check_auto_ivc_warnings
 }
@@ -586,8 +651,12 @@ _all_checks.update({
     'cycles': _check_cycles_prob,
     'unconnected_inputs': _check_hanging_inputs,
     'promotions': _check_explicitly_connected_promoted_inputs,
+    'all_unserializable_options': _check_all_unserializable_options,
 })
 
+_all_non_redundant_checks = _all_checks.copy()
+del _all_non_redundant_checks['unserializable_options']
+
 
 #
 # Command line interface functions
@@ -638,20 +707,19 @@ def _check_config_cmd(options, user_args):
             if not options.checks:
                 options.checks = sorted(_default_checks)
             elif 'all' in options.checks:
-                options.checks = sorted(_all_checks)
+                options.checks = sorted(_all_non_redundant_checks)
 
             prob.check_config(logger, options.checks)
 
-        exit()
-
     # register the hook
-    _register_hook('final_setup', class_name='Problem', inst_id=options.problem, post=_check_config)
+    _register_hook('final_setup', class_name='Problem', inst_id=options.problem, post=_check_config,
+                   exit=True)
 
     ignore_errors(True)
     _load_and_exec(options.file[0], user_args)
 
 
-def check_allocate_complex_ln(model, under_cs):
+def check_allocate_complex_ln(group, under_cs):
     """
     Return True if linear vector should be complex.
 
@@ -659,8 +727,8 @@ def check_allocate_complex_ln(model, under_cs):
 
     Parameters
     ----------
-    model : <Group>
-        Model to be checked, usually the root model.
+    group : <Group>
+        Group to be checked.
     under_cs : bool
         Flag indicates if complex vectors were allocated in a containing Group or were force
         allocated in setup.
@@ -670,16 +738,19 @@ def check_allocate_complex_ln(model, under_cs):
     bool
         True if linear vector should be complex.
     """
-    under_cs |= 'cs' in model._approx_schemes
+    under_cs |= 'cs' in group._approx_schemes
 
-    if under_cs and model.nonlinear_solver is not None and \
-       model.nonlinear_solver.supports['gradients']:
+    if under_cs and group.nonlinear_solver is not None and \
+       group.nonlinear_solver.supports['gradients']:
         return True
 
-    for sub, _ in model._subsystems_allprocs.values():
-        chk = check_allocate_complex_ln(sub, under_cs)
-
-        if chk:
+    for sub, _ in group._subsystems_allprocs.values():
+        if isinstance(sub, Group) and check_allocate_complex_ln(sub, under_cs):
             return True
 
+        elif isinstance(sub, ImplicitComponent):
+            if sub.nonlinear_solver is not None and sub.nonlinear_solver.supports['gradients']:
+                # Special case, gradient-supporting solver in an ImplicitComponent.
+                return True
+
     return False
