@@ -1,6 +1,6 @@
 """Define the AssembledJacobian class."""
 import sys
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 
 import numpy as np
 
@@ -18,6 +18,13 @@ class AssembledJacobian(Jacobian):
     """
     Assemble a global <Jacobian>.
 
+    Parameters
+    ----------
+    matrix_class : type
+        Class to use to create internal matrices.
+    system : System
+        Parent system to this jacobian.
+
     Attributes
     ----------
     _view_ranges : dict
@@ -43,13 +50,6 @@ class AssembledJacobian(Jacobian):
     def __init__(self, matrix_class, system):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        matrix_class : type
-            Class to use to create internal matrices.
-        system : System
-            Parent system to this jacobian.
         """
         global Component
         # avoid circular imports
@@ -78,10 +78,10 @@ class AssembledJacobian(Jacobian):
 
         Returns
         -------
-        OrderedDict
+        dict
             Tuples of the form (start, end) keyed on variable name.
         """
-        ranges = OrderedDict()
+        ranges = {}
         start = end = 0
         for name, meta in system._var_abs2meta[vtype].items():
             end += meta['size']
@@ -220,14 +220,14 @@ class AssembledJacobian(Jacobian):
         conns = {} if isinstance(system, Component) else system._conn_global_abs_in2out
 
         iproc = system.comm.rank
-        sizes = system._var_sizes['linear']['input']
-        abs2idx = system._var_allprocs_abs2idx['linear']
+        sizes = system._var_sizes['input']
+        abs2idx = system._var_allprocs_abs2idx
         in_offset = {n: np.sum(sizes[iproc, :abs2idx[n]]) for n in
                      system._var_abs2meta['input'] if n not in conns}
 
         subjacs_info = self._subjacs_info
 
-        sizes = system._var_sizes['linear']['output']
+        sizes = system._var_sizes['output']
         for s in system.system_iter(recurse=True, include_self=True, typ=Component):
             for res_abs_name, res_meta in s._var_abs2meta['output'].items():
                 res_offset = np.sum(sizes[iproc, :abs2idx[res_abs_name]])
@@ -338,19 +338,19 @@ class AssembledJacobian(Jacobian):
         if ext_mtx is not None:
             ext_mtx._pre_update()
 
-        if self._randomize:
+        if self._randgen:
             for key in iters:
-                int_mtx._update_submat(key, self._randomize_subjac(subjacs[key]['value'], key))
+                int_mtx._update_submat(key, self._randomize_subjac(subjacs[key]['val'], key))
 
             for key in iters_in_ext:
-                ext_mtx._update_submat(key, self._randomize_subjac(subjacs[key]['value'], key))
+                ext_mtx._update_submat(key, self._randomize_subjac(subjacs[key]['val'], key))
         else:
 
             for key in iters:
-                int_mtx._update_submat(key, subjacs[key]['value'])
+                int_mtx._update_submat(key, subjacs[key]['val'])
 
             for key in iters_in_ext:
-                ext_mtx._update_submat(key, subjacs[key]['value'])
+                ext_mtx._update_submat(key, subjacs[key]['val'])
 
         int_mtx._post_update()
 
@@ -394,18 +394,19 @@ class AssembledJacobian(Jacobian):
                     mask = ext_mtx._create_mask_cache(d_inputs)
                     self._mask_caches[(d_inputs._names, mode)] = mask
 
+            dresids = d_residuals.asarray()
+
             if mode == 'fwd':
                 if d_outputs._names:
-                    d_residuals._data += int_mtx._prod(d_outputs._data, mode)
+                    dresids += int_mtx._prod(d_outputs.asarray(), mode)
                 if do_mask:
-                    d_residuals._data += ext_mtx._prod(d_inputs._data, mode, mask=mask)
+                    dresids += ext_mtx._prod(d_inputs.asarray(), mode, mask=mask)
 
             else:  # rev
-                dresids = d_residuals._data
                 if d_outputs._names:
-                    d_outputs._data += int_mtx._prod(dresids, mode)
+                    d_outputs += int_mtx._prod(dresids, mode)
                 if do_mask:
-                    d_inputs._data += ext_mtx._prod(dresids, mode, mask=mask)
+                    d_inputs += ext_mtx._prod(dresids, mode, mask=mask)
 
     def set_complex_step_mode(self, active):
         """
@@ -431,16 +432,16 @@ class AssembledJacobian(Jacobian):
 class DenseJacobian(AssembledJacobian):
     """
     Assemble dense global <Jacobian>.
+
+    Parameters
+    ----------
+    system : System
+        Parent system to this jacobian.
     """
 
     def __init__(self, system):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        system : System
-            Parent system to this jacobian.
         """
         super().__init__(DenseMatrix, system=system)
 
@@ -448,16 +449,16 @@ class DenseJacobian(AssembledJacobian):
 class COOJacobian(AssembledJacobian):
     """
     Assemble sparse global <Jacobian> in Coordinate list format.
+
+    Parameters
+    ----------
+    system : System
+        Parent system to this jacobian.
     """
 
     def __init__(self, system):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        system : System
-            Parent system to this jacobian.
         """
         super().__init__(COOMatrix, system=system)
 
@@ -465,16 +466,16 @@ class COOJacobian(AssembledJacobian):
 class CSRJacobian(AssembledJacobian):
     """
     Assemble sparse global <Jacobian> in Compressed Row Storage format.
+
+    Parameters
+    ----------
+    system : System
+        Parent system to this jacobian.
     """
 
     def __init__(self, system):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        system : System
-            Parent system to this jacobian.
         """
         super().__init__(CSRMatrix, system=system)
 
@@ -482,15 +483,15 @@ class CSRJacobian(AssembledJacobian):
 class CSCJacobian(AssembledJacobian):
     """
     Assemble sparse global <Jacobian> in Compressed Col Storage format.
+
+    Parameters
+    ----------
+    system : System
+        Parent system to this jacobian.
     """
 
     def __init__(self, system):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        system : System
-            Parent system to this jacobian.
         """
         super().__init__(CSCMatrix, system=system)
