@@ -9,7 +9,7 @@ from openmdao.recorders.recording_iteration_stack import Recording
 from openmdao.solvers.linesearch.backtracking import BoundsEnforceLS
 from openmdao.solvers.solver import NonlinearSolver
 from openmdao.utils.class_util import overrides_method
-from openmdao.utils.general_utils import simple_warning
+from openmdao.utils.om_warnings import issue_warning, SetupWarning
 from openmdao.utils.mpi import MPI
 
 
@@ -29,6 +29,11 @@ class BroydenSolver(NonlinearSolver):
     """
     Broyden solver.
 
+    Parameters
+    ----------
+    **kwargs : dict
+        Options dictionary.
+
     Attributes
     ----------
     delta_fxm : ndarray
@@ -59,18 +64,13 @@ class BroydenSolver(NonlinearSolver):
         Flag that becomes True when Broyden detects it needs to recompute the inverse Jacobian.
     """
 
-    SOLVER = 'BROYDEN'
+    SOLVER = 'NL: BROYDEN'
 
     def __init__(self, **kwargs):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        **kwargs : dict
-            options dictionary.
         """
-        super(BroydenSolver, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
         # Slot for linear solver
         self.linear_solver = None
@@ -98,7 +98,7 @@ class BroydenSolver(NonlinearSolver):
         """
         Declare options before kwargs are processed in the init method.
         """
-        super(BroydenSolver, self)._declare_options()
+        super()._declare_options()
 
         self.options.declare('alpha', default=0.4,
                              desc="Value to scale the starting Jacobian, which is Identity. This "
@@ -149,7 +149,7 @@ class BroydenSolver(NonlinearSolver):
         depth : int
             Depth of the current system (already incremented).
         """
-        super(BroydenSolver, self)._setup_solvers(system, depth)
+        super()._setup_solvers(system, depth)
         self._recompute_jacobian = True
         self._computed_jacobians = 0
         iproc = system.comm.rank
@@ -182,10 +182,10 @@ class BroydenSolver(NonlinearSolver):
         if len(states) > 0:
             # User has specified states, so we must size them.
             n = 0
-            sizes = system._var_allprocs_abs2meta
+            meta = system._var_allprocs_abs2meta['output']
 
             for i, name in enumerate(states):
-                size = sizes[prom2abs[name][0]]['global_size']
+                size = meta[prom2abs[name][0]]['global_size']
                 self._idx[name] = (n, n + size)
                 n += size
         else:
@@ -238,7 +238,7 @@ class BroydenSolver(NonlinearSolver):
             msg = "The following states are not covered by a solver, and may have been " + \
                   "omitted from the BroydenSolver 'state_vars': "
             msg += ', '.join(sorted(missing))
-            simple_warning(msg)
+            issue_warning(msg, category=SetupWarning)
 
     def _assembled_jac_solver_iter(self):
         """
@@ -261,7 +261,7 @@ class BroydenSolver(NonlinearSolver):
         type_ : str
             Type of solver to set: 'LN' for linear, 'NL' for nonlinear, or 'all' for all.
         """
-        super(BroydenSolver, self)._set_solver_print(level=level, type_=type_)
+        super()._set_solver_print(level=level, type_=type_)
 
         if self.linear_solver is not None and type_ != 'NL':
             self.linear_solver._set_solver_print(level=level, type_=type_)
@@ -297,10 +297,10 @@ class BroydenSolver(NonlinearSolver):
 
         # Convert local storage if we are under complex step.
         if system.under_complex_step:
-            self.Gm = self.Gm.astype(np.complex)
-            self.xm = self.xm.astype(np.complex)
-            self.fxm = self.fxm.astype(np.complex)
-        elif np.iscomplex(self.xm[0]):
+            self.Gm = self.Gm.astype(complex)
+            self.xm = self.xm.astype(complex)
+            self.fxm = self.fxm.astype(complex)
+        elif np.iscomplexobj(self.xm):
             self.Gm = self.Gm.real
             self.xm = self.xm.real
             self.fxm = self.fxm.real
@@ -315,12 +315,12 @@ class BroydenSolver(NonlinearSolver):
         # to trigger reconvergence, so nudge the outputs slightly so that we always get at least
         # one iteration of Broyden.
         if system.under_complex_step and self.options['cs_reconverge']:
-            system._outputs._data += np.linalg.norm(system._outputs._data) * 1e-10
+            system._outputs += np.linalg.norm(system._outputs.asarray()) * 1e-10
 
         # Start with initial states.
         self.xm = self.get_vector(system._outputs)
 
-        with Recording('Broyden', 0, self):
+        with Recording('Broyden', 0, self) as rec:
             self._solver_info.append_solver()
 
             # should call the subsystems solve before computing the first residual
@@ -328,10 +328,13 @@ class BroydenSolver(NonlinearSolver):
 
             self._solver_info.pop()
 
-        self._run_apply()
-        norm = self._iter_get_norm()
+            self._run_apply()
+            norm = self._iter_get_norm()
+
+            rec.abs = norm
+            norm0 = norm if norm != 0.0 else 1.0
+            rec.rel = norm / norm0
 
-        norm0 = norm if norm != 0.0 else 1.0
         return norm0, norm
 
     def _iter_get_norm(self):
@@ -347,7 +350,7 @@ class BroydenSolver(NonlinearSolver):
         self.fxm = fxm = self.get_vector(self._system()._residuals)
         if not self._full_inverse:
             # Use full model residual for driving the main loop convergence.
-            fxm = self._system()._residuals._data
+            fxm = self._system()._residuals.asarray()
 
         return self.compute_norm(fxm)
 
@@ -396,10 +399,9 @@ class BroydenSolver(NonlinearSolver):
             self.set_states(xm)
 
         # Run the model.
-        with Recording('Broyden', 0, self):
-            self._solver_info.append_solver()
-            self._gs_iter()
-            self._solver_info.pop()
+        self._solver_info.append_solver()
+        self._gs_iter()
+        self._solver_info.pop()
 
         self._run_apply()
 
@@ -490,7 +492,7 @@ class BroydenSolver(NonlinearSolver):
             Array containing values of vector at desired states.
         """
         if self._full_inverse:
-            xm = vec._data.copy()
+            xm = vec.asarray(copy=True)
         else:
             states = self.options['state_vars']
             xm = self.xm.copy()
@@ -512,7 +514,7 @@ class BroydenSolver(NonlinearSolver):
         outputs = self._system()._outputs
 
         if self._full_inverse:
-            outputs._data[:] = new_val
+            outputs.set_val(new_val)
         else:
             states = self.options['state_vars']
             for name in states:
@@ -531,9 +533,9 @@ class BroydenSolver(NonlinearSolver):
         linear = self._system()._vectors['output']['linear']
 
         if self._full_inverse:
-            linear._data[:] = dx
+            linear.set_val(dx)
         else:
-            linear.set_const(0.0)
+            linear.set_val(0.0)
             for name in self.options['state_vars']:
                 i, j = self._idx[name]
                 linear[name] = dx[i:j]
@@ -547,16 +549,16 @@ class BroydenSolver(NonlinearSolver):
         ndarray
             New inverse Jacobian.
         """
-        # TODO : Consider promoting this capability out into OpenMDAO so other solvers can use the
+        # TODO: Consider promoting this capability out into OpenMDAO so other solvers can use the
         # same code.
-        # TODO : Can do each state in parallel if procs are available.
+        # TODO: Can do each state in parallel if procs are available.
         system = self._system()
         states = self.options['state_vars']
         d_res = system._vectors['residual']['linear']
         d_out = system._vectors['output']['linear']
 
         inv_jac = self.Gm
-        d_res.set_const(0.0)
+        d_res.set_val(0.0)
 
         # Disable local fd
         approx_status = system._owns_approx_jac
@@ -583,7 +585,7 @@ class BroydenSolver(NonlinearSolver):
                     d_wrt[j] = 1.0
 
                 # Solve for total derivatives.
-                ln_solver.solve(['linear'], 'fwd')
+                ln_solver.solve('fwd')
 
                 # Extract results.
                 for of_name in states:
@@ -630,27 +632,11 @@ class BroydenSolver(NonlinearSolver):
 
         return inv_jac
 
-    def _mpi_print_header(self):
-        """
-        Print header text before solving.
-        """
-        if self.options['iprint'] > 0 and self._system().comm.rank == 0:
-
-            pathname = self._system().pathname
-            if pathname:
-                nchar = len(pathname)
-                prefix = self._solver_info.prefix
-                header = prefix + "\n"
-                header += prefix + nchar * "=" + "\n"
-                header += prefix + pathname + "\n"
-                header += prefix + nchar * "="
-                print(header)
-
     def cleanup(self):
         """
         Clean up resources prior to exit.
         """
-        super(BroydenSolver, self).cleanup()
+        super().cleanup()
 
         if self.linear_solver:
             self.linear_solver.cleanup()
