@@ -15,6 +15,11 @@ class NewtonSolver(NonlinearSolver):
 
     The default linear solver is the linear_solver in the containing system.
 
+    Parameters
+    ----------
+    **kwargs : dict
+        Options dictionary.
+
     Attributes
     ----------
     linear_solver : LinearSolver
@@ -29,11 +34,6 @@ class NewtonSolver(NonlinearSolver):
     def __init__(self, **kwargs):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        **kwargs : dict
-            options dictionary.
         """
         super().__init__(**kwargs)
 
@@ -147,7 +147,7 @@ class NewtonSolver(NonlinearSolver):
 
         Returns
         -------
-        boolean
+        bool
             Flag for indicating child linerization
         """
         return (self.options['solve_subsystems']
@@ -189,7 +189,8 @@ class NewtonSolver(NonlinearSolver):
         # Execute guess_nonlinear if specified.
         system._guess_nonlinear()
 
-        with Recording('Newton_subsolve', 0, self):
+        with Recording('Newton_subsolve', 0, self) as rec:
+
             if self.options['solve_subsystems'] and \
                (self._iter_count <= self.options['max_sub_solves']):
 
@@ -200,10 +201,13 @@ class NewtonSolver(NonlinearSolver):
 
                 self._solver_info.pop()
 
-        self._run_apply()
-        norm = self._iter_get_norm()
+            self._run_apply()
+            norm = self._iter_get_norm()
+
+            rec.abs = norm
+            norm0 = norm if norm != 0.0 else 1.0
+            rec.rel = norm / norm0
 
-        norm0 = norm if norm != 0.0 else 1.0
         return norm0, norm
 
     def _single_iteration(self):
