@@ -15,6 +15,7 @@ from openmdao.recorders.recording_manager import RecordingManager
 from openmdao.utils.mpi import MPI
 from openmdao.utils.options_dictionary import OptionsDictionary
 from openmdao.utils.record_util import create_local_meta, check_path
+from openmdao.utils.om_warnings import issue_warning, SolverWarning
 from openmdao.core.component import Component
 
 _emptyset = set()
@@ -28,7 +29,7 @@ class SolverInfo(object):
     ----------
     prefix : str
         Prefix to prepend during this iprint.
-    stack : List
+    stack : list
         List of strings; strings are popped and appended as needed.
     """
 
@@ -85,7 +86,7 @@ class SolverInfo(object):
         Returns
         -------
         tuple(str, list)
-            cache of current stack
+            Cache of current stack.
         """
         return (self.prefix, self.stack)
 
@@ -96,7 +97,7 @@ class SolverInfo(object):
         Parameters
         ----------
         cache : tuple(str, list)
-            cache of current stack
+            Cache of current stack.
         """
         self.prefix, self.stack = cache
 
@@ -108,14 +109,17 @@ class Solver(object):
     This class is subclassed by NonlinearSolver and LinearSolver,
     which are in turn subclassed by actual solver implementations.
 
+    Parameters
+    ----------
+    **kwargs : dict of keyword arguments
+        Keyword arguments that will be mapped into the Solver options.
+
     Attributes
     ----------
     _system : <System>
         Pointer to the owning system.
     _depth : int
         How many subsolvers deep this solver is (0 means not a subsolver).
-    _vec_names : [str, ...]
-        List of right-hand-side (RHS) vector names.
     _mode : str
         'fwd' or 'rev', applicable to linear solvers only.
     _iter_count : int
@@ -132,9 +136,9 @@ class Solver(object):
     supports : <OptionsDictionary>
         Options dictionary describing what features are supported by this
         solver.
-    _filtered_vars_to_record: Dict
+    _filtered_vars_to_record : Dict
         Dict of list of var names to record
-    _norm0: float
+    _norm0 : float
         Normalization factor
     _problem_meta : dict
         Problem level metadata.
@@ -146,15 +150,9 @@ class Solver(object):
     def __init__(self, **kwargs):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        **kwargs : dict of keyword arguments
-            Keyword arguments that will be mapped into the Solver options.
         """
         self._system = None
         self._depth = 0
-        self._vec_names = None
         self._mode = 'fwd'
         self._iter_count = 0
         self._problem_meta = None
@@ -299,8 +297,7 @@ class Solver(object):
         if isinstance(self, LinearSolver) and not system._use_derivatives:
             return
 
-        self._rec_mgr.startup(self)
-        self._rec_mgr.record_metadata(self)
+        self._rec_mgr.startup(self, self._problem_meta['comm'])
 
         myoutputs = myresiduals = myinputs = []
         incl = self.recording_options['includes']
@@ -314,13 +311,13 @@ class Solver(object):
             excl = ['.'.join((system.pathname, i)) for i in excl]
 
         if self.recording_options['record_solver_residuals']:
-            myresiduals = [n for n in system._residuals._views if check_path(n, incl, excl)]
+            myresiduals = [n for n in system._residuals._abs_iter() if check_path(n, incl, excl)]
 
         if self.recording_options['record_outputs']:
-            myoutputs = [n for n in system._outputs._views if check_path(n, incl, excl)]
+            myoutputs = [n for n in system._outputs._abs_iter() if check_path(n, incl, excl)]
 
         if self.recording_options['record_inputs']:
-            myinputs = [n for n in system._inputs._views if check_path(n, incl, excl)]
+            myinputs = [n for n in system._inputs._abs_iter() if check_path(n, incl, excl)]
 
         self._filtered_vars_to_record = {
             'input': myinputs,
@@ -356,7 +353,8 @@ class Solver(object):
         rel_res : float
             current relative residual norm.
         """
-        if (self.options['iprint'] == 2 and self._system().comm.rank == 0):
+        if (self.options['iprint'] == 2 and
+                (self._system().comm.rank == 0 or os.environ.get('USE_PROC_FILES'))):
 
             prefix = self._solver_info.prefix
             solver_name = self.SOLVER
@@ -372,78 +370,18 @@ class Solver(object):
         """
         Print header text before solving.
         """
-        pass
+        if (self.options['iprint'] > 0 and
+                (self._system().comm.rank == 0 or os.environ.get('USE_PROC_FILES'))):
 
-    def _solve(self):
-        """
-        Run the iterative solver.
-        """
-        maxiter = self.options['maxiter']
-        atol = self.options['atol']
-        rtol = self.options['rtol']
-        iprint = self.options['iprint']
-
-        self._mpi_print_header()
-
-        self._iter_count = 0
-        norm0, norm = self._iter_initialize()
-
-        self._norm0 = norm0
-
-        self._mpi_print(self._iter_count, norm, norm / norm0)
-
-        while self._iter_count < maxiter and norm > atol and norm / norm0 > rtol:
-            with Recording(type(self).__name__, self._iter_count, self) as rec:
-                self._single_iteration()
-                self._iter_count += 1
-                self._run_apply()
-                norm = self._iter_get_norm()
-                # With solvers, we want to record the norm AFTER the call, but the call needs to
-                # be wrapped in the with for stack purposes, so we locally assign  norm & norm0
-                # into the class.
-                rec.abs = norm
-                if norm0 == 0:
-                    norm0 = 1
-                rec.rel = norm / norm0
-
-            self._mpi_print(self._iter_count, norm, norm / norm0)
-
-        system = self._system()
-        if system.comm.rank == 0 or os.environ.get('USE_PROC_FILES'):
-            prefix = self._solver_info.prefix + self.SOLVER
-
-            # Solver terminated early because a Nan in the norm doesn't satisfy the while-loop
-            # conditionals.
-            if np.isinf(norm) or np.isnan(norm):
-                msg = "Solver '{}' on system '{}': residuals contain 'inf' or 'NaN' after {} " + \
-                      "iterations."
-                if iprint > -1:
-                    print(prefix + msg.format(self.SOLVER, system.pathname,
-                                              self._iter_count))
-
-                # Raise AnalysisError if requested.
-                if self.options['err_on_non_converge']:
-                    raise AnalysisError(msg.format(self.SOLVER, system.pathname,
-                                                   self._iter_count))
-
-            # Solver hit maxiter without meeting desired tolerances.
-            elif (norm > atol and norm / norm0 > rtol):
-                msg = "Solver '{}' on system '{}' failed to converge in {} iterations."
-
-                if iprint > -1:
-                    print(prefix + msg.format(self.SOLVER, system.pathname,
-                                              self._iter_count))
-
-                # Raise AnalysisError if requested.
-                if self.options['err_on_non_converge']:
-                    raise AnalysisError(msg.format(self.SOLVER, system.pathname,
-                                                   self._iter_count))
-
-            # Solver converged
-            elif iprint == 1:
-                print(prefix + ' Converged in {} iterations'.format(self._iter_count))
-            elif iprint == 2:
-                print(prefix + ' Converged')
+            pathname = self._system().pathname
+            if pathname:
+                nchar = len(pathname)
+                prefix = self._solver_info.prefix
+                header = prefix + "\n"
+                header += prefix + nchar * "=" + "\n"
+                header += prefix + pathname + "\n"
+                header += prefix + nchar * "="
+                print(header)
 
     def _iter_initialize(self):
         """
@@ -476,7 +414,7 @@ class Solver(object):
 
         Returns
         -------
-        boolean
+        bool
             Flag for indicating child linerization
         """
         return True
@@ -560,8 +498,7 @@ class Solver(object):
             return
 
         from openmdao.core.group import Group
-        if (isinstance(s, Group) and (s._has_distrib_vars or s._contains_parallel_group)) or \
-           (isinstance(s, Component) and s.options['distributed']):
+        if s._has_distrib_vars or (isinstance(s, Group) and s._contains_parallel_group):
             msg = "{} linear solver in {} cannot be used in or above a ParallelGroup or a " + \
                 "distributed component."
             raise RuntimeError(msg.format(type(self).__name__, s.msginfo))
@@ -571,6 +508,11 @@ class NonlinearSolver(Solver):
     """
     Base class for nonlinear solvers.
 
+    Parameters
+    ----------
+    **kwargs : dict
+        Options dictionary.
+
     Attributes
     ----------
     _err_cache : dict
@@ -580,13 +522,8 @@ class NonlinearSolver(Solver):
     def __init__(self, **kwargs):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        **kwargs : dict
-            options dictionary.
         """
-        super(NonlinearSolver, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self._err_cache = OrderedDict()
 
     def _declare_options(self):
@@ -597,6 +534,14 @@ class NonlinearSolver(Solver):
                              desc='If true, the values of input and output variables at '
                                   'the start of iteration are printed and written to a file '
                                   'after a failure to converge.')
+        self.options.declare('stall_limit', default=0,
+                             desc='Number of iterations after which, if the residual norms are '
+                                  'identical within the stall_tol, then terminate as if max '
+                                  'iterations were reached. Default is 0, which disables this '
+                                  'feature.')
+        self.options.declare('stall_tol', default=1e-12,
+                             desc='When stall checking is enabled, the threshold below which the '
+                                  'residual norm is considered unchanged.')
 
     def solve(self):
         """
@@ -633,6 +578,124 @@ class NonlinearSolver(Solver):
         norm0 = norm if norm != 0.0 else 1.0
         return norm0, norm
 
+    def _solve(self):
+        """
+        Run the iterative solver.
+        """
+        maxiter = self.options['maxiter']
+        atol = self.options['atol']
+        rtol = self.options['rtol']
+        iprint = self.options['iprint']
+        stall_limit = self.options['stall_limit']
+        stall_tol = self.options['stall_tol']
+
+        self._mpi_print_header()
+
+        self._iter_count = 0
+        norm0, norm = self._iter_initialize()
+
+        self._norm0 = norm0
+
+        self._mpi_print(self._iter_count, norm, norm / norm0)
+
+        stalled = False
+        stall_count = 0
+        if stall_limit > 0:
+            stall_norm = norm0
+
+        while self._iter_count < maxiter and norm > atol and norm / norm0 > rtol and not stalled:
+            with Recording(type(self).__name__, self._iter_count, self) as rec:
+
+                if stall_count == 3 and not self.linesearch.options['print_bound_enforce']:
+
+                    self.linesearch.options['print_bound_enforce'] = True
+
+                    if self._system().pathname:
+                        pathname = f"{self._system().pathname}."
+                    else:
+                        pathname = ""
+
+                    msg = (f"Your model has stalled three times and may be violating the bounds. "
+                           f"In the future, turn on print_bound_enforce in your solver options "
+                           f"here: \n{pathname}nonlinear_solver.linesearch.options"
+                           f"['print_bound_enforce']=True. "
+                           f"\nThe bound(s) being violated now are:\n")
+                    issue_warning(msg, category=SolverWarning)
+
+                    self._single_iteration()
+                    self.linesearch.options['print_bound_enforce'] = False
+                else:
+                    self._single_iteration()
+
+                self._iter_count += 1
+                self._run_apply()
+                norm = self._iter_get_norm()
+
+                # Save the norm values in the context manager so they can also be recorded.
+                rec.abs = norm
+                if norm0 == 0:
+                    norm0 = 1
+                rec.rel = norm / norm0
+
+                # Check if convergence is stalled.
+                if stall_limit > 0:
+                    rel_norm = rec.rel
+                    norm_diff = np.abs(stall_norm - rel_norm)
+                    if norm_diff <= stall_tol:
+                        stall_count += 1
+                        if stall_count >= stall_limit:
+                            stalled = True
+                    else:
+                        stall_count = 0
+                        stall_norm = rel_norm
+
+            self._mpi_print(self._iter_count, norm, norm / norm0)
+
+        system = self._system()
+
+        # flag for the print statements. we only print on root if USE_PROC_FILES is not set to True
+        print_flag = system.comm.rank == 0 or os.environ.get('USE_PROC_FILES')
+
+        prefix = self._solver_info.prefix + self.SOLVER
+
+        # Solver terminated early because a Nan in the norm doesn't satisfy the while-loop
+        # conditionals.
+        if np.isinf(norm) or np.isnan(norm):
+            msg = "Solver '{}' on system '{}': residuals contain 'inf' or 'NaN' after {} " + \
+                  "iterations."
+            if iprint > -1 and print_flag:
+                print(prefix + msg.format(self.SOLVER, system.pathname,
+                                          self._iter_count))
+
+            # Raise AnalysisError if requested.
+            if self.options['err_on_non_converge']:
+                raise AnalysisError(msg.format(self.SOLVER, system.pathname,
+                                               self._iter_count))
+
+        # Solver hit maxiter without meeting desired tolerances.
+        # Or solver stalled.
+        elif (norm > atol and norm / norm0 > rtol) or stalled:
+
+            if stalled:
+                msg = "Solver '{}' on system '{}' stalled after {} iterations."
+            else:
+                msg = "Solver '{}' on system '{}' failed to converge in {} iterations."
+
+            if iprint > -1 and print_flag:
+                print(prefix + msg.format(self.SOLVER, system.pathname,
+                                          self._iter_count))
+
+            # Raise AnalysisError if requested.
+            if self.options['err_on_non_converge']:
+                raise AnalysisError(msg.format(self.SOLVER, system.pathname,
+                                               self._iter_count))
+
+        # Solver converged
+        elif iprint == 1 and print_flag:
+            print(prefix + ' Converged in {} iterations'.format(self._iter_count))
+        elif iprint == 2 and print_flag:
+            print(prefix + ' Converged')
+
     def _run_apply(self):
         """
         Run the apply_nonlinear method on the system.
@@ -689,8 +752,8 @@ class NonlinearSolver(Solver):
         Perform a Gauss-Seidel iteration over this Solver's subsystems.
         """
         system = self._system()
-        for isub, subsys in enumerate(system._subsystems_allprocs):
-            system._transfer('nonlinear', 'fwd', isub)
+        for subsys, _ in system._subsystems_allprocs.values():
+            system._transfer('nonlinear', 'fwd', subsys.name)
 
             if subsys._is_local:
                 try:
@@ -705,6 +768,11 @@ class LinearSolver(Solver):
     """
     Base class for linear solvers.
 
+    Parameters
+    ----------
+    **kwargs : dict
+        Options dictionary.
+
     Attributes
     ----------
     _rel_systems : set of str
@@ -716,15 +784,10 @@ class LinearSolver(Solver):
     def __init__(self, **kwargs):
         """
         Initialize all attributes.
-
-        Parameters
-        ----------
-        **kwargs : dict
-            options dictionary.
         """
         self._rel_systems = None
         self._assembled_jac = None
-        super(LinearSolver, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     def _assembled_jac_solver_iter(self):
         """
@@ -764,19 +827,17 @@ class LinearSolver(Solver):
         depth : int
             depth of the current system (already incremented).
         """
-        super(LinearSolver, self)._setup_solvers(system, depth)
+        super()._setup_solvers(system, depth)
         if self.options['assemble_jac'] and not self.supports['assembled_jac']:
             raise RuntimeError("Linear solver %s doesn't support assembled "
                                "jacobians." % self.msginfo)
 
-    def solve(self, vec_names, mode, rel_systems=None):
+    def solve(self, mode, rel_systems=None):
         """
         Run the solver.
 
         Parameters
         ----------
-        vec_names : [str, ...]
-            list of names of the right-hand-side vectors.
         mode : str
             'fwd' or 'rev'.
         rel_systems : set of str
@@ -784,6 +845,79 @@ class LinearSolver(Solver):
         """
         raise NotImplementedError("class %s does not implement solve()." % (type(self).__name__))
 
+    def _solve(self):
+        """
+        Run the iterative solver.
+        """
+        maxiter = self.options['maxiter']
+        atol = self.options['atol']
+        rtol = self.options['rtol']
+        iprint = self.options['iprint']
+
+        self._mpi_print_header()
+
+        self._iter_count = 0
+        norm0, norm = self._iter_initialize()
+
+        self._norm0 = norm0
+
+        self._mpi_print(self._iter_count, norm, norm / norm0)
+
+        while self._iter_count < maxiter and norm > atol and norm / norm0 > rtol:
+            with Recording(type(self).__name__, self._iter_count, self) as rec:
+                self._single_iteration()
+                self._iter_count += 1
+                self._run_apply()
+                norm = self._iter_get_norm()
+
+                # Save the norm values in the context manager so they can also be recorded.
+                rec.abs = norm
+                if norm0 == 0:
+                    norm0 = 1
+                rec.rel = norm / norm0
+
+            self._mpi_print(self._iter_count, norm, norm / norm0)
+
+        system = self._system()
+
+        # flag for the print statements. we only print on root if USE_PROC_FILES is not set to True
+        print_flag = system.comm.rank == 0 or os.environ.get('USE_PROC_FILES')
+
+        prefix = self._solver_info.prefix + self.SOLVER
+
+        # Solver terminated early because a Nan in the norm doesn't satisfy the while-loop
+        # conditionals.
+        if np.isinf(norm) or np.isnan(norm):
+            msg = "Solver '{}' on system '{}': residuals contain 'inf' or 'NaN' after {} " + \
+                  "iterations."
+            if iprint > -1 and print_flag:
+                print(prefix + msg.format(self.SOLVER, system.pathname,
+                                          self._iter_count))
+
+            # Raise AnalysisError if requested.
+            if self.options['err_on_non_converge']:
+                raise AnalysisError(msg.format(self.SOLVER, system.pathname,
+                                               self._iter_count))
+
+        # Solver hit maxiter without meeting desired tolerances.
+        elif (norm > atol and norm / norm0 > rtol):
+            msg = "Solver '{}' on system '{}' failed to converge in {} iterations."
+
+            if iprint > -1 and print_flag:
+                print(prefix + msg.format(self.SOLVER, system.pathname,
+                                          self._iter_count))
+
+            # Raise AnalysisError if requested.
+            if self.options['err_on_non_converge']:
+                raise AnalysisError(msg.format(self.SOLVER, system.pathname,
+                                               self._iter_count))
+
+        # Solver converged
+        elif iprint == 1 and print_flag:
+            print(prefix + ' Converged in {} iterations'.format(self._iter_count))
+        elif iprint == 2 and print_flag:
+            print(prefix + ' Converged')
+
     def _run_apply(self):
         """
         Run the apply_linear method on the system.
@@ -794,7 +928,7 @@ class LinearSolver(Solver):
         scope_out, scope_in = system._get_scope()
 
         try:
-            system._apply_linear(self._assembled_jac, self._vec_names, self._rel_systems,
+            system._apply_linear(self._assembled_jac, self._rel_systems,
                                  self._mode, scope_out, scope_in)
         finally:
             self._recording_iter.pop()
@@ -803,13 +937,18 @@ class LinearSolver(Solver):
 class BlockLinearSolver(LinearSolver):
     """
     A base class for LinearBlockGS and LinearBlockJac.
+
+    Parameters
+    ----------
+    **kwargs : dict
+        Options dictionary.
     """
 
     def _declare_options(self):
         """
         Declare options before kwargs are processed in the init method.
         """
-        super(BlockLinearSolver, self)._declare_options()
+        super()._declare_options()
         self.supports['assembled_jac'] = False
 
     def _setup_solvers(self, system, depth):
@@ -823,26 +962,23 @@ class BlockLinearSolver(LinearSolver):
         depth : int
             depth of the current system (already incremented).
         """
-        super(BlockLinearSolver, self)._setup_solvers(system, depth)
+        super()._setup_solvers(system, depth)
         if system._use_derivatives:
-            self._create_rhs_vecs()
+            self._create_rhs_vec()
 
-    def _create_rhs_vecs(self):
-        self._rhs_vecs = rhs = {}
+    def _create_rhs_vec(self):
         system = self._system()
-        for vec_name in system._lin_rel_vec_name_list:
-            if self._mode == 'fwd':
-                rhs[vec_name] = system._vectors['residual'][vec_name]._data.copy()
-            else:
-                rhs[vec_name] = system._vectors['output'][vec_name]._data.copy()
+        if self._mode == 'fwd':
+            self._rhs_vec = system._vectors['residual']['linear'].asarray(True)
+        else:
+            self._rhs_vec = system._vectors['output']['linear'].asarray(True)
 
-    def _update_rhs_vecs(self):
+    def _update_rhs_vec(self):
         system = self._system()
-        for vec_name in system._lin_rel_vec_name_list:
-            if self._mode == 'fwd':
-                self._rhs_vecs[vec_name][:] = system._vectors['residual'][vec_name]._data
-            else:
-                self._rhs_vecs[vec_name][:] = system._vectors['output'][vec_name]._data
+        if self._mode == 'fwd':
+            self._rhs_vec[:] = system._vectors['residual']['linear'].asarray()
+        else:
+            self._rhs_vec[:] = system._vectors['output']['linear'].asarray()
 
     def _set_complex_step_mode(self, active):
         """
@@ -855,11 +991,10 @@ class BlockLinearSolver(LinearSolver):
         active : bool
             Complex mode flag; set to True prior to commencing complex step.
         """
-        for vec_name in self._system()._lin_rel_vec_name_list:
-            if active:
-                self._rhs_vecs[vec_name] = self._rhs_vecs[vec_name].astype(np.complex)
-            else:
-                self._rhs_vecs[vec_name] = self._rhs_vecs[vec_name].real
+        if active:
+            self._rhs_vec = self._rhs_vec.astype(complex)
+        else:
+            self._rhs_vec = self._rhs_vec.real
 
     def _iter_initialize(self):
         """
@@ -872,7 +1007,7 @@ class BlockLinearSolver(LinearSolver):
         float
             error at the first iteration.
         """
-        self._update_rhs_vecs()
+        self._update_rhs_vec()
         if self.options['maxiter'] > 1:
             self._run_apply()
             norm = self._iter_get_norm()
@@ -896,31 +1031,24 @@ class BlockLinearSolver(LinearSolver):
         system = self._system()
 
         if self._mode == 'fwd':
-            b_vecs = system._vectors['residual']
+            b_vecs = system._vectors['residual']['linear']
         else:  # rev
-            b_vecs = system._vectors['output']
+            b_vecs = system._vectors['output']['linear']
 
-        norm = 0
-        for vec_name in system._lin_rel_vec_name_list:
-            b_vecs[vec_name]._data -= self._rhs_vecs[vec_name]
-            norm += b_vecs[vec_name].get_norm()**2
+        b_vecs -= self._rhs_vec
+        return b_vecs.get_norm()
 
-        return norm ** 0.5
-
-    def solve(self, vec_names, mode, rel_systems=None):
+    def solve(self, mode, rel_systems=None):
         """
         Run the solver.
 
         Parameters
         ----------
-        vec_names : [str, ...]
-            list of names of the right-hand-side vectors.
         mode : str
             'fwd' or 'rev'.
         rel_systems : set of str
             Set of names of relevant systems based on the current linear solve.
         """
-        self._vec_names = vec_names
         self._rel_systems = rel_systems
         self._mode = mode
         self._solve()
