@@ -1,31 +1,28 @@
 """Some miscellaneous utility functions."""
-from contextlib import contextmanager
 import os
 import re
 import sys
 import warnings
 import unittest
+from contextlib import contextmanager
 from fnmatch import fnmatchcase
 from io import StringIO
-from numbers import Number
+from numbers import Integral
 
-# note: this is a Python 3.3 change, clean this up for OpenMDAO 3.x
-try:
-    from collections.abc import Iterable
-except ImportError:
-    from collections import Iterable
-
-import numbers
+from collections.abc import Iterable
 
 import numpy as np
 
-from openmdao.core.constants import INT_DTYPE, INF_BOUND
+from openmdao.core.constants import INF_BOUND
 from openmdao.utils.om_warnings import issue_warning, _warn_simple_format, warn_deprecation
+from openmdao.utils.array_utils import shape_to_len
 
 # Certain command line tools can make use of this to allow visualization of models when errors
 # are present that would normally cause setup to abort.
 _ignore_errors = False
 
+_float_inf = float('inf')
+
 
 def _convert_auto_ivc_to_conn_name(conns_dict, name):
     """
@@ -76,8 +73,10 @@ def conditional_error(msg, exc=RuntimeError, category=UserWarning, err=None):
     ----------
     msg : str
         The error/warning message.
-    exc : Exception class
-        This exception class is used to create the exception to be raised.
+    exc : Exception class or exception info tuple (exception class, exception instance, traceback)
+        This exception class is used to create the exception to be raised, or an exception info
+        tuple from a previously raised exception that is to be re-raised, contingent on the value
+        of 'err'.
     category : warning class
         This category is the class of warning to be issued.
     err : bool
@@ -87,7 +86,10 @@ def conditional_error(msg, exc=RuntimeError, category=UserWarning, err=None):
     if (err is None and ignore_errors()) or err is False:
         issue_warning(msg, category=category)
     else:
-        raise exc(msg)
+        if isinstance(exc, tuple):
+            raise exc[0](msg).with_traceback(exc[2])
+        else:
+            raise exc(msg)
 
 
 @contextmanager
@@ -168,7 +170,7 @@ def ensure_compatible(name, value, shape=None, indices=None):
 
     # if shape is not given, infer from value (if not scalar) or indices
     if shape is not None:
-        if isinstance(shape, numbers.Integral):
+        if isinstance(shape, Integral):
             shape = (shape,)
         elif isinstance(shape, list):
             shape = tuple(shape)
@@ -177,16 +179,16 @@ def ensure_compatible(name, value, shape=None, indices=None):
 
     if indices is not None:
         if not indices._flat_src and shape is None:
-            raise RuntimeError("src_indices for '%s' is not flat, so its input "
-                               "shape must be provided." % name)
+            raise RuntimeError(f"src_indices for '{name}' is not flat, so its input "
+                               "shape must be provided.")
         try:
             indshape = indices.indexed_src_shape
         except (RuntimeError, ValueError, TypeError):
             pass  # use shape provided or shape of value and check vs. shape of indices later
         else:
-            if shape is not None and np.product(indshape) != np.product(shape):
-                raise ValueError("Shape of indices %s does not match shape of %s for '%s'." %
-                                 (indshape, shape, name))
+            if shape is not None and shape_to_len(indshape) != shape_to_len(shape):
+                raise ValueError(f"Shape of indices {indshape} does not match shape of {shape} for"
+                                 f" '{name}'.")
             if shape is None:
                 shape = indshape
 
@@ -197,13 +199,13 @@ def ensure_compatible(name, value, shape=None, indices=None):
     else:
         # shape is determined, if value is scalar assign it to array of shape
         # otherwise make sure value is an array of the determined shape
-        if np.isscalar(value) or value.shape == (1,):
-            value = np.ones(shape) * value
+        if np.ndim(value) == 0 or value.shape == (1,):
+            value = np.full(shape, value)
         else:
             value = np.atleast_1d(value).astype(np.float64)
             if value.shape != shape:
-                raise ValueError("Incompatible shape for '%s': Expected %s but got %s." %
-                                 (name, shape, value.shape))
+                raise ValueError(f"Incompatible shape for '{name}': Expected {shape} but got "
+                                 f"{value.shape}.")
 
     return value, shape
 
@@ -366,26 +368,27 @@ def format_as_float_or_array(name, values, val_if_none=0.0, flatten=False):
         If values is scalar, not None, and not a Number.
     """
     # Convert adder to ndarray/float as necessary
-    if isinstance(values, np.ndarray):
-        if flatten:
-            values = values.flatten()
-    elif not isinstance(values, str) \
-            and isinstance(values, Iterable):
-        values = np.asarray(values, dtype=float)
+    if isinstance(values, float):
+        if values == _float_inf:
+            values = INF_BOUND
+        elif values == -_float_inf:
+            values = -INF_BOUND
+    elif isinstance(values, np.ndarray):
         if flatten:
             values = values.flatten()
     elif values is None:
         values = val_if_none
-    elif values == float('inf'):
-        values = INF_BOUND
-    elif values == -float('inf'):
-        values = -INF_BOUND
-    elif isinstance(values, numbers.Number):
-        values = float(values)
+    elif isinstance(values, Iterable) and not isinstance(values, str):
+        values = np.asarray(values, dtype=float)
+        if flatten:
+            values = values.flatten()
     else:
-        raise TypeError('Expected values of {0} to be an Iterable of '
-                        'numeric values, or a scalar numeric value. '
-                        'Got {1} instead.'.format(name, values))
+        try:
+            values = float(values)
+        except Exception:
+            raise TypeError(f'Expected values of {name} to be an Iterable of '
+                            'numeric values, or a scalar numeric value. '
+                            f'Got {values} instead.')
     return values
 
 
@@ -440,7 +443,7 @@ def find_matches(pattern, var_list):
     Parameters
     ----------
     pattern : str
-        String pattern.
+        Glob pattern or variable name.
     var_list : list of str
         List of variable names to search for pattern.
 
@@ -456,6 +459,28 @@ def find_matches(pattern, var_list):
     return [name for name in var_list if fnmatchcase(name, pattern)]
 
 
+def _find_dict_meta(dct, key):
+    """
+    Return True if the given key is found in any metadata values in the given dict.
+
+    Parameters
+    ----------
+    dct : dict
+        The metadata dictionary (a dict of dicts).
+    key : str
+        The metadata key being searched for.
+
+    Returns
+    -------
+    bool
+        True if non-None metadata at the given key was found.
+    """
+    for meta in dct.values():
+        if key in meta and meta[key] is not None:
+            return True
+    return False
+
+
 def pad_name(name, pad_num=10, quotes=False):
     """
     Pad a string so that they all line up when stacked.
@@ -579,7 +604,8 @@ def printoptions(*args, **kwds):
 
     See Also
     --------
-        set_printoptions, get_printoptions
+        set_printoptions : Set printing options.
+        get_printoptions : Get printing options.
     """
     opts = np.get_printoptions()
 
@@ -890,6 +916,9 @@ def match_prom_or_abs(name, prom_name, includes=None, excludes=None):
     return False
 
 
+_falsey = {'0', 'false', 'no', ''}
+
+
 def env_truthy(env_var):
     """
     Return True if the given environment variable is 'truthy'.
@@ -904,7 +933,7 @@ def env_truthy(env_var):
     bool
         True if the specified environment variable is 'truthy'.
     """
-    return os.environ.get(env_var, '0').lower() not in ('0', 'false', 'no', '')
+    return os.environ.get(env_var, '0').lower() not in _falsey
 
 
 def common_subpath(pathnames):
@@ -964,85 +993,58 @@ def _is_slicer_op(indices):
     return isinstance(indices, slice)
 
 
-def _slice_indices(slicer, arr_size, arr_shape):
+def _src_name_iter(proms):
     """
-    Return an index array based on a slice or slice tuple and the array size and shape.
+    Yield keys from proms with promoted input names converted to source names.
 
     Parameters
     ----------
-    slicer : slice or tuple containing slices
-        Slice object to slice array
-    arr_size : int
-        Size of output array
-    arr_shape : tuple
-        Tuple of output array shape
-
-    Returns
-    -------
-    array
-        Returns the sliced indices.
-    """
-    if isinstance(slicer, slice):
-        # for a simple slice we can use less memory
-        start, stop, step = slicer.start, slicer.stop, slicer.step
-        if start is None:
-            start = 0
-        if stop is None:
-            stop = arr_size
-        if step is None:
-            step = 1
-        return np.arange(start, stop, step, dtype=INT_DTYPE).reshape(arr_shape)
-    else:
-        return np.arange(arr_size, dtype=INT_DTYPE).reshape(arr_shape)[slicer]
-
-
-def _prom2ivc_src_name_iter(prom_dict):
-    """
-    Yield keys from prom_dict with promoted input names converted to ivc source names.
-
-    Parameters
-    ----------
-    prom_dict : dict
+    proms : dict
         Original dict with some promoted paths.
 
     Yields
     ------
     str
-        name
+        source pathname name
     """
-    for name, meta in prom_dict.items():
-        if meta['ivc_source'] is not None:
-            yield meta['ivc_source']
-        else:
-            yield name
+    for meta in proms.values():
+        yield meta['source']
 
 
-def _prom2ivc_src_item_iter(prom_dict):
+def _src_or_alias_name(meta):
+    if 'alias' in meta:
+        alias = meta['alias']
+        if alias:
+            return alias
+    return meta['source']
+
+
+def _src_or_alias_item_iter(proms):
     """
-    Yield items from prom_dict with promoted input names converted to ivc source names.
-
-    The result is that all names are absolute.
+    Yield items from proms with promoted input names converted to source or alias names.
 
     Parameters
     ----------
-    prom_dict : dict
+    proms : dict
         Original dict with some promoted paths.
 
     Yields
     ------
     tuple
-        name, metadata
+        src_or_alias_name, metadata
     """
-    for name, meta in prom_dict.items():
-        if meta['ivc_source'] is not None:
-            yield meta['ivc_source'], meta
+    for name, meta in proms.items():
+        if 'alias' in meta and meta['alias'] is not None:
+            yield meta['alias'], meta
+        elif meta['source'] is not None:
+            yield meta['source'], meta
         else:
             yield name, meta
 
 
-def _prom2ivc_src_dict(prom_dict):
+def _src_or_alias_dict(prom_dict):
     """
-    Convert a dictionary with promoted input names into one with ivc source names.
+    Convert a dict with promoted input names into one with source or alias names.
 
     Parameters
     ----------
@@ -1052,9 +1054,9 @@ def _prom2ivc_src_dict(prom_dict):
     Returns
     -------
     dict
-        New dict with ivc source pathnames.
+        New dict with source pathnames or alias names.
     """
-    return {name: meta for name, meta in _prom2ivc_src_item_iter(prom_dict)}
+    return {name: meta for name, meta in _src_or_alias_item_iter(prom_dict)}
 
 
 def convert_src_inds(parent_src_inds, parent_src_shape, my_src_inds, my_src_shape):
@@ -1099,14 +1101,22 @@ def shape2tuple(shape):
 
     Returns
     -------
-    tuple
-        The shape as a tuple.
+    tuple or None
+        The shape as a tuple or None if shape is None.
     """
-    if isinstance(shape, Number):
+    if isinstance(shape, tuple):
+        return shape
+    elif isinstance(shape, int):
         return (shape,)
     elif shape is None:
         return shape
-    return tuple(shape)
+    else:
+        try:
+            return tuple(shape)
+        except TypeError:
+            if not isinstance(shape, Integral):
+                raise TypeError(f"{type(shape).__name__} is not a valid shape type.")
+            return (shape,)
 
 
 def get_connection_owner(system, tgt):
@@ -1125,7 +1135,7 @@ def get_connection_owner(system, tgt):
     Returns
     -------
     tuple
-        (wning group, promoted source name, promoted target name).
+        (owning group, promoted source name, promoted target name).
     """
     from openmdao.core.group import Group
 
@@ -1140,9 +1150,9 @@ def get_connection_owner(system, tgt):
                 if g._manual_connections:
                     tprom = g._var_allprocs_abs2prom['input'][tgt]
                     if tprom in g._manual_connections:
-                        return g.pathname, g._var_allprocs_abs2prom['output'][src], tprom
+                        return g, g._var_allprocs_abs2prom['output'][src], tprom
 
-    return None, None, None
+    return system, src, tgt
 
 
 def wing_dbg():
