@@ -7,16 +7,29 @@ import os
 import argparse
 from openmdao import __version__ as version
 
+try:
+    import coverage
+except ImportError:
+    pass
+else:
+    # the following allows us to do coverage on our command line tools.  It will do nothing
+    # unless COVERAGE_PROCESS_START is set in the environment.
+    try:
+        coverage.process_startup()
+    except Exception:
+        pass  # in case they're using an old version of coverage
+
 try:
     import pkg_resources
 except ImportError:
     pkg_resources = None
 
-from itertools import chain
-
 import openmdao.utils.hooks as hooks
 from openmdao.visualization.n2_viewer.n2_viewer import n2
 from openmdao.visualization.connection_viewer.viewconns import view_connections
+from openmdao.visualization.scaling_viewer.scaling_report import _scaling_setup_parser, \
+    _scaling_cmd
+from openmdao.visualization.timing_viewer.timing_viewer import _timing_setup_parser, _timing_cmd
 from openmdao.visualization.dyn_shape_plot import _view_dyn_shapes_setup_parser, \
     _view_dyn_shapes_cmd
 try:
@@ -24,8 +37,10 @@ try:
     from openmdao.visualization.meta_model_viewer.meta_model_visualization import view_metamodel
 except ImportError:
     bokeh = None
-from openmdao.components.meta_model_unstructured_comp import MetaModelUnStructuredComp
+from openmdao.components.meta_model_semi_structured_comp import MetaModelSemiStructuredComp
 from openmdao.components.meta_model_structured_comp import MetaModelStructuredComp
+from openmdao.components.meta_model_unstructured_comp import MetaModelUnStructuredComp
+from openmdao.core.component import Component
 from openmdao.devtools.debug import config_summary, tree
 from openmdao.devtools.itrace import _itrace_exec, _itrace_setup_parser
 from openmdao.devtools.iprofile_app.iprofile_app import _iprof_exec, _iprof_setup_parser
@@ -45,8 +60,8 @@ from openmdao.utils.file_utils import _load_and_exec, _to_filename
 from openmdao.utils.entry_points import _list_installed_setup_parser, _list_installed_cmd, \
     split_ep, _compute_entry_points_setup_parser, _compute_entry_points_exec, \
         _find_plugins_setup_parser, _find_plugins_exec
-from openmdao.core.component import Component
-from openmdao.utils.general_utils import ignore_errors, warn_deprecation
+from openmdao.utils.general_utils import ignore_errors
+from openmdao.utils.om_warnings import warn_deprecation
 
 
 def _n2_setup_parser(parser):
@@ -58,7 +73,10 @@ def _n2_setup_parser(parser):
     parser : argparse subparser
         The parser we're adding options to.
     """
-    parser.add_argument('file', nargs=1, help='Python script or recording containing the model.')
+    parser.add_argument('file', nargs=1,
+                        help='Python script or recording containing the model. '
+                        'If metadata from a parallel run was recorded in a separate file, '
+                        'specify both database filenames delimited with a comma.')
     parser.add_argument('-o', default='n2.html', action='store', dest='outfile',
                         help='html output file.')
     parser.add_argument('--no_browser', action='store_true', dest='no_browser',
@@ -97,10 +115,13 @@ def _n2_cmd(options, user_args):
         def _viewmod(prob):
             n2(prob, outfile=options.outfile, show_browser=not options.no_browser,
                 title=options.title, embeddable=options.embeddable)
-            exit()  # could make this command line selectable later
 
-        hooks._register_hook('setup', 'Problem', pre=_noraise)
-        hooks._register_hook('final_setup', 'Problem', post=_viewmod)
+        hooks._register_hook('setup', 'Problem', pre=_noraise, ncalls=1)
+        hooks._register_hook('final_setup', 'Problem', post=_viewmod, exit=True)
+
+        from openmdao.utils.reports_system import _register_cmdline_report
+        # tell report system not to duplicate effort
+        _register_cmdline_report('n2')
 
         ignore_errors(True)
         _load_and_exec(options.file[0], user_args)
@@ -149,14 +170,14 @@ def _view_connections_cmd(options, user_args):
             title = "Connections for %s" % os.path.basename(options.file[0])
         view_connections(prob, outfile=options.outfile, show_browser=not options.no_browser,
                          show_values=options.show_values, title=title)
-        exit()
 
     # register the hook
     if options.show_values:
         funcname = 'final_setup'
     else:
         funcname = 'setup'
-    hooks._register_hook(funcname, class_name='Problem', inst_id=options.problem, post=_viewconns)
+    hooks._register_hook(funcname, class_name='Problem', inst_id=options.problem, post=_viewconns,
+                         exit=True)
 
     ignore_errors(True)
     _load_and_exec(options.file[0], user_args)
@@ -202,7 +223,8 @@ def _meta_model_cmd(options, user_args):
 
         hooks._unregister_hook('final_setup', 'Problem')
 
-        mm_types = (MetaModelStructuredComp, MetaModelUnStructuredComp)
+        mm_types = (MetaModelStructuredComp, MetaModelUnStructuredComp,
+                    MetaModelSemiStructuredComp)
 
         pathname = options.pathname
         port_number = options.port_number
@@ -223,6 +245,10 @@ def _meta_model_cmd(options, user_args):
         mm_names = list(metamodels.keys())
         mm_count = len(mm_names)
 
+        def _mm_list(mm_names, options):
+            for mm in mm_names:
+                print("openmdao view_mm -m {} {}".format(mm, options.file[0]))
+
         if mm_count == 0:
             print("No Metamodel components found in model.")
 
@@ -231,17 +257,26 @@ def _meta_model_cmd(options, user_args):
             view_metamodel(comp, resolution, port_number, browser)
 
         else:
-            try_str = "Try one of the following: {}.".format(mm_names)
-
             if not pathname:
-                print("\nMetamodel not specified. {}".format(try_str))
+                print("\nMetamodel not specified. Try one of the following:\n")
+                _mm_list(mm_names, options)
             elif not comp:
-                print("\nMetamodel '{}' not found.\n {}".format(pathname, try_str))
+                if len(mm_names) > 1:
+                    print("\nMetamodel '{}' not found. Try one of the "
+                          "following:\n".format(pathname))
+                else:
+                    print("\nMetamodel '{}' not found. Try the "
+                          "following:\n".format(pathname))
+                _mm_list(mm_names, options)
             else:
-                print("\n'{}' is not a Metamodel.\n {}".format(pathname, try_str))
-        exit()
+                if len(mm_names) > 1:
+                    print("\n'{}' is not a Metamodel. Try one of the "
+                          "following:\n".format(pathname))
+                else:
+                    print("\n'{}' is not a Metamodel. Try the following:\n".format(pathname))
+                _mm_list(mm_names, options)
 
-    hooks._register_hook('final_setup', 'Problem', post=_view_metamodel)
+    hooks._register_hook('final_setup', 'Problem', post=_view_metamodel, exit=True)
 
     _load_and_exec(options.file[0], user_args)
 
@@ -269,11 +304,7 @@ def _config_summary_cmd(options, user_args):
     user_args : list of str
         Args to be passed to the user script.
     """
-    def summary(prob):
-        config_summary(prob)
-        sys.exit(0)
-
-    hooks._register_hook('final_setup', 'Problem', post=summary)
+    hooks._register_hook('final_setup', 'Problem', post=config_summary, exit=True)
 
     ignore_errors(True)
     _load_and_exec(options.file[0], user_args)
@@ -377,14 +408,14 @@ def _tree_cmd(options, user_args):
         tree(prob, show_colors=options.show_colors, show_sizes=options.show_sizes,
              show_approx=options.show_approx, filter=filt, max_depth=options.depth,
              rank=options.rank, stream=out)
-        exit()
 
     # register the hook
     if options.vecvars or options.show_sizes or options.show_approx:
         funcname = 'final_setup'
     else:
         funcname = 'setup'
-    hooks._register_hook(funcname, class_name='Problem', inst_id=options.problem, post=_tree)
+    hooks._register_hook(funcname, class_name='Problem', inst_id=options.problem, post=_tree,
+                         exit=True)
 
     ignore_errors(True)
     _load_and_exec(options.file[0], user_args)
@@ -428,9 +459,8 @@ def _cite_cmd(options, user_args):
     def _cite(prob):
         if not MPI or MPI.COMM_WORLD.rank == 0:
             print_citations(prob, classes=options.classes, out_stream=out)
-        exit()
 
-    hooks._register_hook('setup', 'Problem', post=_cite)
+    hooks._register_hook('setup', 'Problem', post=_cite, exit=True)
 
     ignore_errors(True)
     _load_and_exec(options.file[0], user_args)
@@ -465,6 +495,7 @@ _command_map = {
                  'Generate a simple scaffold for a component.'),
     'summary': (_config_summary_setup_parser, _config_summary_cmd,
                 'Print a short top-level summary of the problem.'),
+    'timing': (_timing_setup_parser, _timing_cmd, 'Collect timing information for all systems.'),
     'total_coloring': (_total_coloring_setup_parser, _total_coloring_cmd,
                        'Compute a coloring for the total jacobian.'),
     'trace': (_itrace_setup_parser, _itrace_exec, 'Dump trace output.'),
@@ -474,7 +505,8 @@ _command_map = {
                          'View connections showing values and source/target units.'),
     'view_dyn_shapes': (_view_dyn_shapes_setup_parser, _view_dyn_shapes_cmd,
                         'View the dynamic shape dependency graph.'),
-    'view_mm': (_meta_model_parser, _meta_model_cmd, "View a metamodel.")
+    'view_mm': (_meta_model_parser, _meta_model_cmd, "View a metamodel."),
+    'scaling': (_scaling_setup_parser, _scaling_cmd, 'View driver scaling report.'),
 }
 
 
@@ -492,10 +524,12 @@ def openmdao_cmd():
         user_args = []
 
     parser = argparse.ArgumentParser(description='OpenMDAO Command Line Tools',
-                                     epilog='Use -h after any sub-command for sub-command help.'
-                                     ' If using a tool on a script that takes its own command line'
-                                     ' arguments, place those arguments after a "--". For example:'
-                                     ' openmdao n2 -o foo.html myscript.py -- -x --myarg=bar')
+                                     epilog='Use -h after any sub-command for sub-command help, '
+                                     'for example, "openmdao tree -h" for help on the "tree" '
+                                     'command. If using a tool on a script that takes its own '
+                                     'command line arguments, place those arguments after a "--". '
+                                     'For example: '
+                                     '"openmdao n2 -o foo.html myscript.py -- -x --myarg=bar"')
 
     parser.add_argument('--version', action='version', version=version)
 
@@ -535,7 +569,8 @@ def openmdao_cmd():
 
     # handle case where someone just runs `openmdao <script> [dashed-args]`
     args = [a for a in sys.argv[1:] if not a.startswith('-')]
-    if not set(args).intersection(subs.choices) and len(args) == 1 and os.path.isfile(args[0]):
+    cmdargs = [a for a in sys.argv[1:] if a not in ('-h', '--version')]
+    if not set(args).intersection(subs.choices) and len(args) == 1 and os.path.isfile(cmdargs[0]):
         _load_and_exec(args[0], user_args)
     else:
         hooks.use_hooks = True
