@@ -1,9 +1,17 @@
 """Define the OptionsDictionary class."""
 
+import re
+
 from openmdao.utils.om_warnings import warn_deprecation
+from openmdao.utils.notebook_utils import notebook_mode
+
 from openmdao.core.constants import _UNDEFINED
 
 
+# regex to check for valid names.
+namecheck_rgx = re.compile('[a-zA-Z_][_a-zA-Z0-9]*')
+
+
 #
 # Template for `check_valid` function
 #
@@ -53,8 +61,8 @@ class OptionsDictionary(object):
         If True, no options can be set after declaration.
     _all_recordable : bool
         Flag to determine if all options in UserOptions are recordable.
-    _deprecation_warning_issued : list
-        Option names that are deprecated and a warning has been issued for their use.
+    _widget : OptionsWidget
+        If running in a Jupyter notebook, a widget for viewing/setting options.
     """
 
     def __init__(self, parent_name=None, read_only=False):
@@ -64,11 +72,8 @@ class OptionsDictionary(object):
         self._dict = {}
         self._parent_name = parent_name
         self._read_only = read_only
-
         self._all_recordable = True
 
-        self._deprecation_warning_issued = []
-
     def __getstate__(self):
         """
         Return state as a dict.
@@ -96,6 +101,15 @@ class OptionsDictionary(object):
         """
         return self._dict.__repr__()
 
+    def _repr_pretty_(self, p, cycle):
+        if not cycle and notebook_mode():
+            try:
+                from openmdao.visualization.options_widget import OptionsWidget
+                return OptionsWidget(self)
+            except Exception:
+                pass
+        return repr(self)
+
     def __rst__(self):
         """
         Generate reStructuredText view of the options table.
@@ -130,43 +144,51 @@ class OptionsDictionary(object):
             from tabulate import tabulate
         except ImportError as e:
             msg = "'to_table' requires the tabulate package but it is not currently installed." \
-                  " Use `pip install tablulate` or install openmdao with" \
+                  " Use `pip install tabulate` or install openmdao with" \
                   " `pip install openmdao[notebooks]`."
             raise ImportError(msg)
 
-        tlist = [['Option', 'Default', 'Acceptable Values', 'Acceptable Types', 'Description',
-                  'Deprecation']]
+        hdrs = ['Option', 'Default', 'Acceptable Values', 'Acceptable Types', 'Description']
+        rows = []
+
         for key in sorted(self._dict.keys()):
-            options = self._dict[key]
-            default = options['val'] if options['val'] is not _UNDEFINED else '**Required**'
-            # if the default is an object instance, replace with the (unqualified) object type
+            option = self._dict[key]
+            deprecations = False
+
+            default = option['val'] if option['val'] is not _UNDEFINED else '**Required**'
             default_str = str(default)
+
+            # if the default is an object instance, replace with the (unqualified) object type
             idx = default_str.find(' object at ')
             if idx >= 0 and default_str[0] == '<':
                 parts = default_str[:idx].split('.')
                 default = parts[-1]
 
-            acceptable_values = options['values']
+            acceptable_values = option['values']
             if acceptable_values is not None:
                 if not isinstance(acceptable_values, (set, tuple, list)):
                     acceptable_values = (acceptable_values,)
                 acceptable_values = [value for value in acceptable_values]
 
-            acceptable_types = options['types']
+            acceptable_types = option['types']
             if acceptable_types is not None:
                 if not isinstance(acceptable_types, (set, tuple, list)):
                     acceptable_types = (acceptable_types,)
                 acceptable_types = [type_.__name__ for type_ in acceptable_types]
 
-            desc = options['desc']
+            desc = option['desc']
 
-            deprecation = options['deprecation']
+            deprecation = option['deprecation']
             if deprecation is not None:
-                tlist.append([key, default, acceptable_values, acceptable_types, desc, deprecation])
+                if not deprecations:
+                    hdrs.append('Deprecation')
+                    deprecations = True
+                rows.append([key, default, acceptable_values, acceptable_types, desc,
+                             deprecation[0]])
             else:
-                tlist.append([key, default, acceptable_values, acceptable_types, desc])
+                rows.append([key, default, acceptable_values, acceptable_types, desc])
 
-        return tabulate(tlist, headers='firstrow', tablefmt=fmt, missingval=missingval)
+        return tabulate(rows, headers=hdrs, tablefmt=fmt, missingval=missingval)
 
     def __str__(self, width=100):
         """
@@ -246,12 +268,14 @@ class OptionsDictionary(object):
         if not (value is None and meta['allow_none']):
             # If only values is declared
             if values is not None:
-                if value not in values:
-                    if isinstance(value, str):
-                        value = "'{}'".format(value)
-                    self._raise("Value ({}) of option '{}' is not one of {}.".format(value, name,
-                                                                                     values),
-                                ValueError)
+                check_vals = [value] if types is not list else value
+                for val in check_vals:
+                    if val not in values:
+                        if isinstance(value, str):
+                            value = f"'{value}'"
+                        self._raise(f"Value ({value}) of option '{name}' is not one of {values}.",
+                                    ValueError)
+
             # If only types is declared
             elif types is not None:
                 if not isinstance(value, types):
@@ -320,20 +344,28 @@ class OptionsDictionary(object):
             If True, allow None as a value regardless of values or types.
         recordable : bool
             If True, add to recorder.
-        deprecation : str or None
+        deprecation : str or tuple or None
             If None, it is not deprecated. If a str, use as a DeprecationWarning
-            during __setitem__ and __getitem__.
+            during __setitem__ and __getitem__.  If a tuple of the form (msg, new_name),
+            display msg as with str, and forward any __setitem__/__getitem__ to new_name.
         """
+        match = namecheck_rgx.match(name)
+        if match is None or match.group() != name:
+            warn_deprecation(f"'{name}' is not a valid python name and will become an invalid "
+                             "option name in a future release. You can prevent this warning (and "
+                             "future exceptions) by declaring this option using a valid python "
+                             "name.")
+
         if values is not None and not isinstance(values, (set, list, tuple)):
-            self._raise("In declaration of option '%s', the 'values' arg must be of type None,"
-                        " list, or tuple - not %s." % (name, values), exc_type=TypeError)
+            self._raise(f"In declaration of option '{name}', the 'values' arg must be of type None,"
+                        f" list, or tuple - not {values}.", exc_type=TypeError)
 
         if types is not None and not isinstance(types, (type, set, list, tuple)):
-            self._raise("In declaration of option '%s', the 'types' arg must be None, a type "
-                        "or a tuple - not %s." % (name, types), exc_type=TypeError)
+            self._raise(f"In declaration of option '{name}', the 'types' arg must be None, a type "
+                        f"or a tuple - not {types}.", exc_type=TypeError)
 
-        if types is not None and values is not None:
-            self._raise("'types' and 'values' were both specified for option '%s'." % name)
+        if types is not None and types is not list and values is not None:
+            self._raise(f"'types' and 'values' were both specified for option '{name}'.")
 
         if types is bool:
             values = (True, False)
@@ -347,6 +379,18 @@ class OptionsDictionary(object):
             # specifying default=None implies allow_none
             allow_none = True
 
+        alias = None
+        if deprecation is not None:
+            if isinstance(deprecation, (list, tuple)):
+                if len(deprecation) != 2:
+                    self._raise("deprecation must be None, str, or a tuple or list containing "
+                                "(str, str).", RuntimeError)
+                dep, alias = deprecation
+                # [message, alias, display warning (becomes False after first display)]
+                deprecation = [dep, alias, True]
+            else:
+                deprecation = [deprecation, None, True]
+
         self._dict[name] = {
             'val': default,
             'values': values,
@@ -430,16 +474,15 @@ class OptionsDictionary(object):
         try:
             meta = self._dict[name]
         except KeyError:
-            # The key must have been declared.
-            msg = "Option '{}' cannot be set because it has not been declared."
-            self._raise(msg.format(name), exc_type=KeyError)
-
-        if meta['deprecation'] is not None and name not in self._deprecation_warning_issued:
-            warn_deprecation(meta['deprecation'])
-            self._deprecation_warning_issued.append(name)
+            # The key must not have been declared.
+            self._raise(f"Option '{name}' cannot be set because it has not been declared.",
+                        exc_type=KeyError)
 
         if self._read_only:
-            self._raise("Tried to set read-only option '{}'.".format(name), exc_type=KeyError)
+            self._raise(f"Tried to set read-only option '{name}'.", exc_type=KeyError)
+
+        if meta['deprecation'] is not None:
+            name, meta = self._handle_deprecation(name, meta)
 
         self._assert_valid(name, value)
 
@@ -463,15 +506,16 @@ class OptionsDictionary(object):
         # If the option has been set in this system, return the set value
         try:
             meta = self._dict[name]
-            if meta['deprecation'] is not None and name not in self._deprecation_warning_issued:
-                warn_deprecation(meta['deprecation'])
-                self._deprecation_warning_issued.append(name)
-            if meta['has_been_set']:
-                return meta['val']
-            else:
-                self._raise("Option '{}' is required but has not been set.".format(name))
         except KeyError:
-            self._raise("Option '{}' cannot be found".format(name), exc_type=KeyError)
+            self._raise(f"Option '{name}' has not been declared.", exc_type=KeyError)
+
+        if meta['deprecation'] is not None:
+            name, meta = self._handle_deprecation(name, meta)
+
+        if meta['has_been_set']:
+            return meta['val']
+        else:
+            self._raise(f"Option '{name}' is required but has not been set.")
 
     def items(self):
         """
@@ -489,3 +533,37 @@ class OptionsDictionary(object):
                 yield key, val['val']
             except KeyError:
                 yield key, val['value']
+
+    def _handle_deprecation(self, name, meta):
+        """
+        Update the warning counter and do name translation of deprecated variable if needed.
+
+        Parameters
+        ----------
+        name : str
+            Name of the deprecated variable.
+        meta : dict
+            Metadata dictionary corresponding to the deprecated variable.
+
+        Returns
+        -------
+        str
+            The variable name, updated to the non-deprecated name if found.
+        dict
+            Metadata dictionary corresponding to either the original variable or to the
+            non-deprecated varsion if found.
+        """
+        msg, alias, show_warn = meta['deprecation']
+        if show_warn:
+            warn_deprecation(msg)
+            meta['deprecation'][2] = False  # turn off future warnings for this variable
+
+        if alias:
+            try:
+                meta = self._dict[alias]
+            except KeyError:
+                msg = f"Can't find aliased option '{alias}' for deprecated option '{name}'."
+                self._raise(msg, exc_type=KeyError)
+            name = alias
+
+        return name, meta
