@@ -4,7 +4,7 @@ import os
 import weakref
 
 import numpy as np
-from numpy import isscalar
+from numpy import ndim
 
 from openmdao.utils.name_maps import prom_name2abs_name, rel_name2abs_name
 from openmdao.utils.indexer import Indexer, indexer
@@ -220,6 +220,26 @@ class Vector(object):
                 if n in self._names:
                     yield v.real
 
+    def items(self):
+        """
+        Return (name, value) for variables contained in this vector.
+
+        Yields
+        ------
+        str
+            Name of each variable.
+        ndarray or float
+            Value of each variable.
+        """
+        if self._under_complex_step:
+            for n, v in self._views.items():
+                if n in self._names:
+                    yield n, v
+        else:
+            for n, v in self._views.items():
+                if n in self._names:
+                    yield n, v.real
+
     def _name2abs_name(self, name):
         """
         Map the given promoted or relative name to the absolute name.
@@ -549,14 +569,14 @@ class Vector(object):
         arr = self.asarray()
 
         if self.nvars() == 1:
-            if isscalar(vals):
+            if ndim(vals) == 0:
                 arr[:] = vals
             else:
                 arr[:] = vals.ravel()
         else:
             start = end = 0
             for v in vals:
-                if isscalar(v):
+                if ndim(v) == 0:
                     end += 1
                     arr[start] = v
                 else:
